
#include <stdio.h>
#include <unistd.h>

#include "impulse/util/main.h"
#include "impulse/util/error.h"
#include "impulse/comms/main.h"
#include "impulse/util/event.h"


impulse_bool client_event_handler(impulse_event *event,
				  void *data)
{
  impulse_client client = (impulse_client)data;
  
  if (event->data.fd.type == IMPULSE_EVENT_FD_READ)
  {
    impulse_int pending = impulse_client_pending(client);
    impulse_int got = 0;
    impulse_char buffer[1024];
    
    if (pending <= 0)
    {
      impulse_quit();
      return 0;
    }
    
    /* Scale down bytes read to a manageable size */
    if (pending > 1023)
      pending = 1023;
      
    got = impulse_client_read(client, buffer, pending);
    if (got <= 0)
    {
      impulse_quit();
      return 0;
    }
    
    buffer[got] = '\0';
    printf(buffer);
  }
  else if (event->data.fd.type == IMPULSE_EVENT_FD_EXCEPTION)
  {
    printf("Exception\n");
    impulse_quit();
    return 0;
  }
  return 1;

}


impulse_bool console_event_handler(impulse_event *event,
				   void *data)
{  
  impulse_client client = (impulse_client)data;
  impulse_char buffer[1024];
  impulse_int offset = 0;
  
  for(;;)
  {
    char c;
    
    if (read(event->data.fd.fd, &c, 1) <= 0)
    {
      impulse_quit();
      return 0;
    }
    
    buffer[offset++] = c;
    if (offset == 1022)
    {
      c = '\n';
      buffer[offset++] = c;
    }
    if (c == '\n')
    {
      buffer[offset++] = '\0';
      impulse_client_write(client, buffer, strlen(buffer));
      break;
    }
  }
  return 1;
}



impulse_int main (impulse_int argc,
		  impulse_char *argv[])
{
  impulse_client client = NULL;
  impulse_unistring local;
  impulse_unistring remote;
  impulse_unistring help;
  impulse_unistring arg;
  impulse_command_value value_local = NULL;
  impulse_command_value value_remote = NULL;
  impulse_bool bind = 0;
  
  impulse_init(argc, argv);
  
  local = impulse_unistring_create_utf("local", 8);
  help = impulse_unistring_create_utf("bind to local address [name]", 8);
  arg = impulse_unistring_create_utf("name", 8);
  impulse_command_option_arg(local, 0, arg, IMPULSE_COMMAND_TYPE_STRING, help);
  impulse_unistring_destroy(help);
  
  remote = impulse_unistring_create_utf("remote", 8);
  help = impulse_unistring_create_utf("connect to remote address [name]", 8);
  impulse_command_option_arg(remote, 0, arg, IMPULSE_COMMAND_TYPE_STRING, help);
  impulse_unistring_destroy(help);
  impulse_unistring_destroy(arg);

  impulse_command_parse();
  impulse_comms_init();
  
  if (!impulse_command_set(remote, &value_remote))
    impulse_command_help();
  
  if (impulse_command_set(local, &value_local))
    bind = 1;
  impulse_unistring_destroy(local);
  impulse_unistring_destroy(remote);
  
  if (impulse_comms_client(bind ? value_local->s : NULL, 
			   value_remote->s, &client) == -1)
  {
    impulse_unistring desc = impulse_unistring_create_utf("Cannot create client", 8);
    impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, value_remote->s);
  }
  
  impulse_event_register_fd(impulse_client_handle(client), IMPULSE_EVENT_FD_READ |
			    IMPULSE_EVENT_FD_EXCEPTION,
			    client_event_handler, client);
  impulse_event_register_fd(STDIN_FILENO, IMPULSE_EVENT_FD_READ, console_event_handler, client);
  
  impulse_main();

  
  impulse_client_close(client);
  impulse_client_destroy(client);
  
  impulse_comms_cleanup();
  
  return 0;
}
