
#include <stdio.h>
#include <unistd.h>

#include "impulse/util/main.h"
#include "impulse/util/error.h"
#include "impulse/util/memory.h"
#include "impulse/comms/main.h"
#include "impulse/util/event.h"
#include <errno.h>

impulse_bool peer_event_handler(impulse_event *event,
				void *data)
{
  impulse_peer peer = (impulse_peer)data;

  if (event->data.fd.type == IMPULSE_EVENT_FD_READ)
  {
    impulse_int pending = impulse_peer_pending(peer);
    impulse_int i;

    if (pending <= 0)
    {
      impulse_quit();
      return 0;
    }
    
    for (i = 0 ; i < pending ; i++)
    {
      impulse_message message = NULL;
      impulse_unistring sender = NULL;
      impulse_char *send = NULL;
      impulse_int got = 0;
      
      got = impulse_peer_receive(peer, &message, &sender);

      if (got < 0)
      {
	impulse_quit();
	return 0;
      }
      
      send = impulse_unistring_to_utf(sender, 8, 0);
      printf("Received message from %s (%d bytes)\n", send, got);
      printf("%s", message->body);
      impulse_free(send);
      impulse_unistring_destroy(sender);
      impulse_message_destroy(message);
    }
  }
  else if (event->data.fd.type == IMPULSE_EVENT_FD_EXCEPTION)
  {
    printf("Exception\n");
    impulse_quit();
    return 0;
  }
  return 1;
}


impulse_bool console_event_handler(impulse_event *event,
				   void *data)
{  
  impulse_peer peer = (impulse_peer)data;
  impulse_char mesg[1024];
  impulse_char dest[1024];
  impulse_bool addr = 1;
  impulse_int offset = 0;

  for(;;)
  {
    char c;
    
    if (read(event->data.fd.fd, &c, 1) <= 0)
    {
      impulse_quit();
      return 0;
    }
    
    if (c == ' ' && addr)
    {
      dest[offset++] = '\0';
      offset = 0;
      addr = 0;
      continue;
    }
    
    if (addr)
      dest[offset++] = c;
    else
      mesg[offset++] = c;
    
    if (offset == 1022)
    {
      c = '\n';
      if (addr)
	dest[offset++] = c;
      else
	mesg[offset++] = c;
    }
    
    if (c == '\n')
    {
      impulse_message message = NULL;
      impulse_unistring to = NULL;
      if (addr)
      {
	printf("syntax: address message\n");
	return 1;
      }
      
      mesg[offset++] = '\0';
      message = impulse_message_create(0,0,strlen(mesg)+1);
      to = impulse_unistring_create_utf(dest, 8);
      strcpy(message->body, mesg);
      if (impulse_peer_send(peer, message, to) < 0)
      {
	printf("COuld not send message to host %d\n", errno);
	impulse_quit();
	impulse_unistring_destroy(to);
	impulse_message_destroy(message);
	return 0;
      }
      impulse_unistring_destroy(to);
      impulse_message_destroy(message);
      break;
    }
  }
  return 1;
}


impulse_int main (impulse_int argc,
		  impulse_char *argv[])
{
  impulse_peer peer = NULL;
  impulse_unistring address;
  impulse_unistring help;
  impulse_unistring arg;
  impulse_command_value value;
  
  impulse_init(argc, argv);
  
  address = impulse_unistring_create_utf("address", 8);
  help = impulse_unistring_create_utf("bind to local address [name]", 8);
  arg = impulse_unistring_create_utf("name", 8);
  impulse_command_option_arg(address, 0, arg, IMPULSE_COMMAND_TYPE_STRING, help);
  impulse_unistring_destroy(arg);
  impulse_unistring_destroy(help);
  
  impulse_command_parse();
  
  if (!impulse_command_set(address, &value))
    impulse_command_help();
  impulse_unistring_destroy(address);
  
  impulse_comms_init();
  
  if (impulse_comms_peer(value->s, &peer) == -1)
  {
    impulse_unistring desc = impulse_unistring_create_utf("Cannot create peer", 8);
    impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, NULL);
  }
  
  impulse_event_register_fd(impulse_peer_handle(peer), IMPULSE_EVENT_FD_READ |
			    IMPULSE_EVENT_FD_EXCEPTION,
			    peer_event_handler, peer);
  impulse_event_register_fd(STDIN_FILENO, IMPULSE_EVENT_FD_READ, console_event_handler, peer);
  
  impulse_main();

  impulse_peer_close(peer);
  impulse_peer_destroy(peer);
  
  impulse_comms_cleanup();
  
  return 0;
}
