
#include <stdio.h>

#include "impulse/util/main.h"
#include "impulse/util/error.h"
#include "impulse/comms/main.h"
#include "impulse/util/event.h"
#include "impulse/util/memory.h"
#include "impulse/util/event.h"
//#include "impulse_hash.h"


//impulse_hash clients;

void client_die(impulse_client client)
{
  impulse_unistring desc = impulse_unistring_create_utf("Connection to client lost", 8);
  impulse_unistring msg = NULL;
  
  impulse_client_address_remote(client, &msg);
  
  impulse_error_raise(IMPULSE_ERROR_INFO, desc, msg);

//  impulse_hash_remove(impulse_client_handle(client));
  
  impulse_client_close(client);
  impulse_client_destroy(client);

  impulse_unistring_destroy(msg);
  impulse_unistring_destroy(desc);
}



impulse_bool client_event_handler(impulse_event *event,
				 void *data)
{
  impulse_client client = (impulse_client)data;
  
  if (event->data.fd.type == IMPULSE_EVENT_FD_READ)
  {
    impulse_int pending = impulse_client_pending(client);
    impulse_int got = 0;
    impulse_char buffer[1024];
    impulse_char *reply = "A reply from the Impulse test server\n";
    
    if (pending <= 0)
    {
      client_die(client);
      return 0;
    }
    
    /* Scale down bytes read to a manageable size */
    if (pending > 1023)
      pending = 1023;
      
    got = impulse_client_read(client, buffer, pending);
    if (got <= 0)
    {
      client_die(client);
      return 0;
    }
    
    buffer[got] = '\0';
    printf(buffer);
    impulse_client_write(client, reply, strlen(reply));
  }
  else if (event->data.fd.type == IMPULSE_EVENT_FD_EXCEPTION)
  {
    printf("Exception\n");
    client_die(client);
    return 0;
  }
  return 1;
}


impulse_bool server_event_handler(impulse_event *event,
				 void *data)
{
  impulse_server server = (impulse_server)data;
  impulse_client client;
  
  if (impulse_server_accept(server, &client) < 0)
  {
    impulse_unistring desc = impulse_unistring_create_utf("Cannot accept incoming connection", 8);
    impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, NULL);
  }
  else
  {
    impulse_unistring desc = impulse_unistring_create_utf("Connection from client", 8);
    impulse_unistring msg = NULL;
    impulse_char *message = "A message from the Impulse test server\n";
    
    impulse_client_write(client, message, strlen(message));
    
//    impulse_hash_store
    
    impulse_event_register_fd(impulse_client_handle(client), IMPULSE_EVENT_FD_READ |
			      IMPULSE_EVENT_FD_EXCEPTION,
			      client_event_handler, client);


    impulse_client_address_remote(client, &msg);
    impulse_error_raise(IMPULSE_ERROR_INFO, desc, msg);
    
    impulse_unistring_destroy(msg);
    impulse_unistring_destroy(desc);
  }
  return 1;
}


impulse_int main (impulse_int argc, 
		  impulse_char *argv[])
{
  impulse_server server = NULL;
  impulse_unistring address;
  impulse_unistring help;
  impulse_command_value value;
  impulse_unistring arg;
  
  impulse_init(argc, argv);
  
  address = impulse_unistring_create_utf("address", 8);
  help = impulse_unistring_create_utf("bind to local address [name]", 8);
  arg = impulse_unistring_create_utf("name", 8);
  impulse_command_option_arg(address, 0, arg, IMPULSE_COMMAND_TYPE_STRING, help);
  impulse_unistring_destroy(help);
  impulse_unistring_destroy(arg);
  
  impulse_command_parse();
  
  if (!impulse_command_set(address, &value))
    impulse_command_help();
  impulse_unistring_destroy(address);
  
  impulse_comms_init();
  
  if (impulse_comms_server(value->s, &server) == -1)
  {
    impulse_unistring desc = impulse_unistring_create_utf("Cannot create server", 8);
    impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, value->s);
  }

  impulse_event_register_fd(impulse_server_handle(server), IMPULSE_EVENT_FD_READ,
			    server_event_handler, server);
  
  impulse_main();
  
  impulse_server_close(server);
  impulse_server_destroy(server);
  
  impulse_comms_cleanup();
  impulse_cleanup();
  
  return 0;
}
