/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * bitmap_test.c
 * -------------
 *
 * A test of bitmaps
 */

#include "impulse/rows/bitmap.h"
#include "impulse/util/memory.h"

#include "bitmap_test.h"

/*                            ======--------======                           */

static void bitmap_test_setup(ROWS_widget *widget);
static int  bitmap_test_ID   = -1;
static int  bitmap_test_type = -1;

static void bitmap_test_destroy(void **private);

static int  bitmap_test_redraw(ROWS_child            widget,
			       ROWS_child_event_num  event_number,
			       ROWS_child_event     *event,
			       void                     *data);
static int  bitmap_test_enter (ROWS_child            widget,
			       ROWS_child_event_num  event_number,
			       ROWS_child_event     *event,
			       void                     *data);
static int  bitmap_test_leave (ROWS_child            widget,
			       ROWS_child_event_num  event_number,
			       ROWS_child_event     *event,
			       void                     *data);

/*                            ======--------======                           */

/* Standard widget functions */

ROWS_widget bitmap_test_create(void)
{
  ROWS_widget *wid;

  if (bitmap_test_ID   == -1)
    bitmap_test_ID   = ROWS_child_next_identifier();

  wid = bitmap_test_initialise(bitmap_test_ID);

  return *wid;
}

ROWS_widget *bitmap_test_initialise(int id)
{
  ROWS_widget *widget;

  if (bitmap_test_ID   == -1)
    bitmap_test_ID   = ROWS_child_next_identifier();
  if (bitmap_test_type == -1)
    bitmap_test_type = ROWS_widget_next_type();

  widget = ROWS_child_initialise(id);
  ROWS_widget_set_type(widget, bitmap_test_ID, bitmap_test_type);
  ROWS_widget_set_destructor(widget, bitmap_test_ID, bitmap_test_destroy);

  bitmap_test_setup(widget);

  return widget;
}

int bitmap_test_next_identifier(void)
{
  if (bitmap_test_ID==-1)
    bitmap_test_ID = ROWS_child_next_identifier();

  return bitmap_test_ID + 1;
}

bitmap_test bitmap_test_cast(void **widget)
{
  if (bitmap_test_ID   == -1)
    bitmap_test_ID   = ROWS_child_next_identifier();
  if (bitmap_test_type == -1)
    bitmap_test_type = ROWS_widget_next_type();

  return ROWS_widget_cast_misc(widget, bitmap_test_ID, bitmap_test_type);
}

struct bitmap_test {
  ROWS_bitmap bitmap;
};

void bitmap_test_setup(ROWS_widget *widget)
{
  struct bitmap_test **private;
  impulse_int             *data;
  impulse_unistring tfile;
  int x,y;

  private  = (struct bitmap_test **) ROWS_widget_private(widget, bitmap_test_ID);
  *private = impulse_alloc(sizeof(struct bitmap_test));

  (*private)->bitmap = ROWS_bitmap_create(PIXEL_TO_MICRONS(20),
					  PIXEL_TO_MICRONS(20),
					  32,
					  0);

  tfile = impulse_unistring_create_utf(PREFIX_DIR, 8);
  impulse_unistring_append_utf(tfile, "/demos/rows/test.png",8);
  ROWS_bitmap_load(ROWS_bitmap_cast((*private)->bitmap), tfile);
  impulse_unistring_destroy(tfile);
  
  data = ROWS_bitmap_get_data(ROWS_bitmap_cast((*private)->bitmap));

  /* Draw a colour square (red in top left, white in bottom right) */
  for (y=0; y<20; y++)
    for (x=0; x<20; x++)
      {
	data[x+(y*21)] = 255|((x*12)<<8)|((y*12)<<16);
      }

  ROWS_child_register_event(ROWS_child_cast(*widget),
			    ROWS_CHILD_REDRAW,
			    bitmap_test_redraw,
			    NULL);
  ROWS_child_register_event(ROWS_child_cast(*widget),
			    ROWS_CHILD_POINTER_ENTER,
			    bitmap_test_enter,
			    NULL);
  ROWS_child_register_event(ROWS_child_cast(*widget),
			    ROWS_CHILD_POINTER_LEAVE,
			    bitmap_test_leave,
			    NULL);
}

void bitmap_test_destroy(void **private)
{
  /* impulse_free(*private); */
}

int bitmap_test_enter(ROWS_child            widget,
		      ROWS_child_event_num  event_number,
		      ROWS_child_event     *event,
		      void                 *data)
{
  ROWS_child_set_activation(widget,
			    ROWS_STYLE_HIGHLIGHT);

  return 0;
}
     
int bitmap_test_leave(ROWS_child            widget,
		      ROWS_child_event_num  event_number,
		      ROWS_child_event     *event,
		      void                 *data)
{
  ROWS_child_set_activation(widget,
			    ROWS_STYLE_NORMAL);

  return 0;
}
     

int bitmap_test_redraw(ROWS_child            widget,
		       ROWS_child_event_num  event_number,
		       ROWS_child_event     *event,
		       void                 *data)
{
  ROWS_drawable            drawable;
  ROWS_style              style;
  ROWS_rect                box;
  ROWS_style_active_type   type;
  struct bitmap_test      **private;

  private = (struct bitmap_test **) bitmap_test_cast(widget);
  type = ROWS_child_get_activation(widget);

  drawable = ROWS_child_get_drawable(widget);

  style = ROWS_child_get_style(widget);
  box.size = *ROWS_box_get_virtual (ROWS_box_cast(widget));
  box.pos  = *ROWS_child_get_origin(ROWS_child_cast(widget));

  ROWS_brush_draw_line(style->brush[type][ROWS_STYLE_BORDERSHADOW],
		       box.pos.x+box.size.x-PIXEL_MULTIPLIER,
		       box.pos.y,
		       box.pos.x+box.size.x-PIXEL_MULTIPLIER,
		       box.pos.y+box.size.y-1);
  ROWS_brush_draw_line(style->brush[type][ROWS_STYLE_BORDERSHADOW],
		       box.pos.x+box.size.x-PIXEL_MULTIPLIER,
		       box.pos.y+box.size.y-PIXEL_MULTIPLIER,
		       box.pos.x,
		       box.pos.y+box.size.y-PIXEL_MULTIPLIER);
  ROWS_brush_draw_line(style->brush[type][ROWS_STYLE_BORDERHILITE],
		       box.pos.x, box.pos.y+box.size.y-PIXEL_MULTIPLIER,
		       box.pos.x, box.pos.y);
  ROWS_brush_draw_line(style->brush[type][ROWS_STYLE_BORDERHILITE],
		       box.pos.x, box.pos.y,
		       box.pos.x+box.size.x-PIXEL_MULTIPLIER, box.pos.y);

  ROWS_drawable_draw_bitmap(drawable,
			    ROWS_bitmap_cast((*private)->bitmap),
			    box.pos.x+(box.size.x/2)-5000,
			    box.pos.y+(box.size.y/2)-5000);
  return 1;
}
