/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * main.c
 * ------
 *
 * Test that widget set!
 */

#include "impulse/rows/main.h"

#include "impulse/rows/constructor.h"
#include "impulse/rows/window.h"

#include "impulse/util/main.h"
#include "impulse/util/event.h"
#include "impulse/util/thread.h"

#include "gl_test.h"

impulse_int main(impulse_int argc,
		 impulse_char *argv[])
{
  ROWS_widget window;
  ROWS_widget layout1;
  ROWS_widget layout2;
  ROWS_widget child1;
  ROWS_widget child2;

  /* Start things off */
  ROWS_init(argc, argv);
  impulse_command_parse();

  /* The main window */
  window = ROWS_window_create("A guidelines test");
//  ROWS_window_thread(ROWS_window_cast(window));
  /* FIXME: Drink. Girls! */
  ROWS_window_set_class(ROWS_window_cast(window), "fek off");
  ROWS_window_set_icon_name(ROWS_window_cast(window), "FatherJack");
  ROWS_child_show(ROWS_child_cast(window));

  /* The main constructor */
  layout1 = ROWS_constructor_create();
  ROWS_container_add(ROWS_container_cast(window),
		     ROWS_child_cast(layout1),
		     4,
		     0);
  ROWS_child_show(ROWS_child_cast(layout1));

  /* Create a few guidelines */
  ROWS_constructor_add_GL(ROWS_constructor_cast(layout1),
			  ROWS_CONSTRUCTOR_VERTICAL,
			  ROWS_CONSTRUCTOR_RELATIVE,
			  0, 1,
			  0.3333);
  ROWS_constructor_add_GL(ROWS_constructor_cast(layout1),
			  ROWS_CONSTRUCTOR_VERTICAL,
			  ROWS_CONSTRUCTOR_RELATIVE,
			  0, 1,
			  0.666);
  ROWS_constructor_add_GL(ROWS_constructor_cast(layout1),
			  ROWS_CONSTRUCTOR_HORIZONTAL,
			  ROWS_CONSTRUCTOR_RELATIVE,
			  2, 3,
			  0.25);
  ROWS_constructor_add_GL(ROWS_constructor_cast(layout1),
			  ROWS_CONSTRUCTOR_VERTICAL,
			  ROWS_CONSTRUCTOR_ABSOLUTE,
			  4, 0,
			  MM_TO_MICRONS(1));
  ROWS_constructor_add_GL(ROWS_constructor_cast(layout1),
			  ROWS_CONSTRUCTOR_HORIZONTAL,
			  ROWS_CONSTRUCTOR_RELATIVE,
			  6, 3,
			  .25);
  ROWS_constructor_add_GL(ROWS_constructor_cast(layout1),
			  ROWS_CONSTRUCTOR_HORIZONTAL,
			  ROWS_CONSTRUCTOR_ABSOLUTE,
			  8, 0,
			  MM_TO_MICRONS(3));
  ROWS_constructor_add_GL(ROWS_constructor_cast(layout1),
			  ROWS_CONSTRUCTOR_HORIZONTAL,
			  ROWS_CONSTRUCTOR_RELATIVE,
			  9, 3,
			  .5);


  /* The nested constructor */
  layout2 = ROWS_constructor_create();
  ROWS_constructor_attach_widget(ROWS_constructor_cast(layout1),
				 ROWS_child_cast(layout2),
				 7, 5, 9, 10);
  ROWS_child_show(ROWS_child_cast(layout2));
  
  /* Create a few guidelines */
  ROWS_constructor_add_GL(ROWS_constructor_cast(layout2),
			  ROWS_CONSTRUCTOR_HORIZONTAL,
			  ROWS_CONSTRUCTOR_RELATIVE,
			  2, 3,
			  .5);
  ROWS_constructor_add_GL(ROWS_constructor_cast(layout2),
			  ROWS_CONSTRUCTOR_VERTICAL,
			  ROWS_CONSTRUCTOR_RELATIVE,
			  0, 1,
			  .5);

  /* Add some contents to the constructors */
  child1 = gl_test_create();
  ROWS_constructor_attach_widget(ROWS_constructor_cast(layout1),
				 ROWS_child_cast(child1),
				 4, 5, 6,
				 8);
  ROWS_child_show(ROWS_child_cast(child1));
  ROWS_child_actual_widget(ROWS_child_cast(child1));


  /* Add another gadget */
  child2 = gl_test_create();
  ROWS_constructor_attach_widget(ROWS_constructor_cast(layout2),
				 ROWS_child_cast(child2),
				 0, 5, 2, 4);
  ROWS_child_show(ROWS_child_cast(child2));

  /* Do a little event processing */
  impulse_main();

  /* Good people always clean up after themselves */
  ROWS_cleanup();
  
  return 0;
}

/* End of file */
