/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * gl_test.c
 * -------------
 *
 * A test of bitmaps
 */

#include "impulse/rows/bitmap.h"
#include "impulse/util/memory.h"

#include "gl_test.h"

/*                            ======--------======                           */

static void gl_test_setup(ROWS_widget *widget);
static int  gl_test_ID   = -1;
static int  gl_test_type = -1;

static void gl_test_destroy(void **private);

static int  gl_test_redraw(ROWS_child            widget,
			       ROWS_child_event_num  event_number,
			       ROWS_child_event     *event,
			       void                     *data);
static int  gl_test_click (ROWS_child            widget,
			       ROWS_child_event_num  event_number,
			       ROWS_child_event     *event,
			       void                     *data);
static int  gl_test_enter (ROWS_child            widget,
			       ROWS_child_event_num  event_number,
			       ROWS_child_event     *event,
			       void                     *data);
static int  gl_test_leave (ROWS_child            widget,
			       ROWS_child_event_num  event_number,
			       ROWS_child_event     *event,
			       void                     *data);

/*                            ======--------======                           */

/* Standard widget functions */

ROWS_widget gl_test_create(void)
{
  ROWS_widget *wid;

  if (gl_test_ID   == -1)
    gl_test_ID   = ROWS_child_next_identifier();

  wid = gl_test_initialise(gl_test_ID);

  return *wid;
}

ROWS_widget *gl_test_initialise(int id)
{
  ROWS_widget *widget;

  if (gl_test_ID   == -1)
    gl_test_ID   = ROWS_child_next_identifier();
  if (gl_test_type == -1)
    gl_test_type = ROWS_widget_next_type();

  widget = ROWS_child_initialise(id);
  ROWS_widget_set_type(widget, gl_test_ID, gl_test_type);
  ROWS_widget_set_destructor(widget, gl_test_ID, gl_test_destroy);

  gl_test_setup(widget);

  return widget;
}

int gl_test_next_identifier(void)
{
  if (gl_test_ID==-1)
    gl_test_ID = ROWS_child_next_identifier();

  return gl_test_ID + 1;
}

gl_test gl_test_cast(void **widget)
{
  if (gl_test_ID   == -1)
    gl_test_ID   = ROWS_child_next_identifier();
  if (gl_test_type == -1)
    gl_test_type = ROWS_widget_next_type();

  return ROWS_widget_cast_misc(widget, gl_test_ID, gl_test_type);
}

struct gl_test {
  impulse_bool down;
};

void gl_test_setup(ROWS_widget *widget)
{
  struct gl_test **private;

  private  = (struct gl_test **) ROWS_widget_private(widget, gl_test_ID);
  *private = impulse_alloc(sizeof(struct gl_test));

  (*private)->down = 0;
  
  ROWS_child_register_event(ROWS_child_cast(*widget),
			    ROWS_CHILD_REDRAW,
			    gl_test_redraw,
			    NULL);
  ROWS_child_register_event(ROWS_child_cast(*widget),
			    ROWS_CHILD_CLICK,
			    gl_test_click,
			    NULL);
  ROWS_child_register_event(ROWS_child_cast(*widget),
			    ROWS_CHILD_POINTER_ENTER,
			    gl_test_enter,
			    NULL);
  ROWS_child_register_event(ROWS_child_cast(*widget),
			    ROWS_CHILD_POINTER_LEAVE,
			    gl_test_leave,
			    NULL);
}

void gl_test_destroy(void **private)
{
  /* impulse_free(*private); */
}

int gl_test_enter(ROWS_child            widget,
		  ROWS_child_event_num  event_number,
		  ROWS_child_event     *event,
		  void                 *data)
{
  ROWS_child_set_activation(widget,
			    ROWS_STYLE_HIGHLIGHT);

  return 0;
}
     
int gl_test_leave(ROWS_child            widget,
		  ROWS_child_event_num  event_number,
		  ROWS_child_event     *event,
		  void                 *data)
{
  ROWS_child_set_activation(widget,
			    ROWS_STYLE_NORMAL);

  return 0;
}
     
int gl_test_click(ROWS_child            widget,
		  ROWS_child_event_num  event_number,
		  ROWS_child_event     *event,
		  void                 *data)
{
  struct gl_test **private;
  
  private  = (struct gl_test **)gl_test_cast(widget);

  if (event->event.click.released)
    (*private)->down = 0;
  else
    (*private)->down = 1;
  
  ROWS_child_update(ROWS_child_cast(widget));
  return 1;
}

int gl_test_redraw(ROWS_child            widget,
		       ROWS_child_event_num  event_number,
		       ROWS_child_event     *event,
		       void                 *data)
{
  ROWS_drawable            drawable;
  ROWS_style              style;
  ROWS_rect                box;
  ROWS_style_active_type   type;
  struct gl_test      **private;
  impulse_bool down;
  
  private = (struct gl_test **) gl_test_cast(widget);
  type = ROWS_child_get_activation(widget);

  down = (*private)->down;
  drawable = ROWS_child_get_drawable(widget);

  style = ROWS_child_get_style(widget);
  box.size = *ROWS_box_get_virtual (ROWS_box_cast(widget));
  box.pos  = *ROWS_child_get_origin(ROWS_child_cast(widget));

  ROWS_brush_draw_line(style->brush[type][down ? ROWS_STYLE_BORDERHILITE : ROWS_STYLE_BORDERSHADOW],
		       box.pos.x+box.size.x-PIXEL_MULTIPLIER,
		       box.pos.y,
		       box.pos.x+box.size.x-PIXEL_MULTIPLIER,
		       box.pos.y+box.size.y-1);
  ROWS_brush_draw_line(style->brush[type][down ? ROWS_STYLE_BORDERHILITE : ROWS_STYLE_BORDERSHADOW],
		       box.pos.x+box.size.x-PIXEL_MULTIPLIER,
		       box.pos.y+box.size.y-PIXEL_MULTIPLIER,
		       box.pos.x,
		       box.pos.y+box.size.y-PIXEL_MULTIPLIER);
  ROWS_brush_draw_line(style->brush[type][down ? ROWS_STYLE_BORDERSHADOW : ROWS_STYLE_BORDERHILITE],
		       box.pos.x, box.pos.y+box.size.y-PIXEL_MULTIPLIER,
		       box.pos.x, box.pos.y);
  ROWS_brush_draw_line(style->brush[type][down ? ROWS_STYLE_BORDERSHADOW : ROWS_STYLE_BORDERHILITE],
		       box.pos.x, box.pos.y,
		       box.pos.x+box.size.x-PIXEL_MULTIPLIER, box.pos.y);

  return 1;
}
