/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * test_mqueue.c
 * -------------
 *
 * Test the dynamic module loading interface
 */

#include <stdio.h>

#include "impulse/util/main.h"
#include "impulse/util/module.h"
#include "impulse/util/unistring.h"


impulse_int main(impulse_int argc, 
		 impulse_char *argv[])
{
  impulse_unistring modname = NULL;
  impulse_unistring cosfunc = NULL;
  impulse_module mod = NULL;
  void *cosf;
  
  impulse_init(argc, argv);
  impulse_command_parse();
  
  modname = impulse_unistring_create_utf("/usr/lib/libm", 8);
  cosfunc =impulse_unistring_create_utf("cos", 8);
  mod = impulse_module_create(modname);
  
  printf("Lets see if we can use the maths library\n");
  
  if (mod)
  {
    cosf = impulse_module_symbol(mod, cosfunc);
    
    if (cosf)
    {
      double (* cosfn)(double) = cosf;
      
      printf("Cos of 1.4 is %f\n", (*cosfn)(1.4));
    }
    else
      printf("Huh? Your maths library doesn't have a cos function\n");
    impulse_module_destroy(mod);
  }
  else
    printf("Nope, it's probably not where we are looking\n");
  impulse_unistring_destroy(modname);
  impulse_unistring_destroy(cosfunc);


  printf("\n\nI wonder if GTK wants to come and play...\n");
  /* 
   * Who says, we can't play nice with gtk :)
   */
  {
    impulse_unistring glibname = impulse_unistring_create_utf("/usr/lib/libglib", 8);
    impulse_unistring gdkname = impulse_unistring_create_utf("/usr/lib/libgdk", 8);
    impulse_unistring gtkname = impulse_unistring_create_utf("/usr/lib/libgtk", 8);

    impulse_unistring gtkinitfunc = impulse_unistring_create_utf("gtk_init", 8);
    impulse_unistring gtkwinnewfunc = impulse_unistring_create_utf("gtk_window_new", 8);
    impulse_unistring gtkwinshowfunc = impulse_unistring_create_utf("gtk_widget_show", 8);
    impulse_unistring gtkmainfunc = impulse_unistring_create_utf("gtk_main", 8);
    impulse_module glibmod = impulse_module_create(glibname);
    impulse_module gdkmod = impulse_module_create(gdkname);
    impulse_module gtkmod = impulse_module_create(gtkname);
    
    if (glibmod && gdkmod && gtkmod)
    {
      void (*gtk_init)(int *,char**) = impulse_module_symbol(gtkmod, gtkinitfunc);
      void *(*gtk_new)(int) = impulse_module_symbol(gtkmod, gtkwinnewfunc);
      void (*gtk_show)(void *) = impulse_module_symbol(gtkmod, gtkwinshowfunc);
      void (*gtk_main)(void) = impulse_module_symbol(gtkmod, gtkmainfunc);
      
      if (gtk_init && gtk_new && gtk_show && gtk_main)
      {
	int i = 0;
	void *win;
	(*gtk_init)(&i, NULL);
	win = (*gtk_new)(1);
	(*gtk_show)(win);
	printf("Ohhhh, yes it does\n");
	printf("You'll probably have to press Control-C to terminate this\n");
	(*gtk_main)();
      }
      else
	printf("No, it doesn't know how to initialise itself\n");
    }
    else
      printf("No, its feeling moody\n");
    if (glibmod)
      impulse_module_destroy(glibmod);
    if (gdkmod)
      impulse_module_destroy(gdkmod);
    if (gtkmod)
      impulse_module_destroy(gtkmod);
    
    impulse_unistring_destroy(glibname);
    impulse_unistring_destroy(gtkname);
    impulse_unistring_destroy(gdkname);
    impulse_unistring_destroy(gtkinitfunc);
    impulse_unistring_destroy(gtkwinnewfunc);
    impulse_unistring_destroy(gtkwinshowfunc);
    impulse_unistring_destroy(gtkmainfunc);
  }
  printf("Aren't dynamic module fun!\n");

  impulse_cleanup();
  
  return 0;
}


/* End of file */
