/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * channel.c
 * ---------
 *
 * Communication channel functions
 */


#include "../../../config.h"

#include "impulse/comms/channel.h"
#include "impulse/util/memory.h"
#include "impulse/util/queue.h"


struct __p_impulse_channel
{
  impulse_byte *start;
  impulse_byte *middle;
  impulse_byte *end;
  
  impulse_message message;
  
  impulse_queue queue;
};


typedef struct __p_impulse_channel _p_impulse_channel;

impulse_channel impulse_channel_create(impulse_client client)
{
  impulse_channel chan = impulse_alloc(sizeof(struct _impulse_channel));
  
  chan->client = client;
  chan->internal = impulse_alloc(sizeof(struct __p_impulse_channel));
  chan->internal->start = NULL;
  chan->internal->middle = NULL;
  chan->internal->end = NULL;
  chan->internal->message = NULL;
  chan->internal->queue = impulse_queue_create();
  impulse_queue_cleaner(chan->internal->queue,
			(impulse_queue_cleaner_func)impulse_message_destroy);
  
  chan->name = impulse_unistring_create_empty();
  chan->release = impulse_unistring_create_empty();
  chan->vendor = impulse_unistring_create_empty();
  chan->platform = impulse_unistring_create_empty();
  chan->architecture = impulse_unistring_create_empty();
  
  return chan;
}


void impulse_channel_destroy(impulse_channel chan)
{
  impulse_client_destroy(chan->client);
  impulse_unistring_destroy(chan->name);
  impulse_unistring_destroy(chan->release);
  impulse_unistring_destroy(chan->vendor);
  impulse_unistring_destroy(chan->platform);
  impulse_unistring_destroy(chan->architecture);
  
  if (chan->internal->start)
    impulse_free(chan->internal->start);
  impulse_queue_destroy(chan->internal->queue);
  impulse_free(chan->internal);
  impulse_free(chan);
}


impulse_int _p_impulse_channel_process(impulse_channel chan,
				       impulse_int amount)
{
  impulse_int need = chan->internal->end - chan->internal->middle;
  need = (need > amount) ? amount : need;
  
  impulse_client_read(chan->client, chan->internal->middle, need);
  chan->internal->middle += need;
  
  if (chan->internal->middle == chan->internal->end)
  {
    if (!chan->internal->message) /* We're reading a header */
    {
    }
    else /* We're reading a payload */
    {
    }
  }
  
  return need;
}


impulse_int impulse_channel_pending(impulse_channel chan)
{
  impulse_int pending = impulse_client_pending(chan->client);
  if (!pending)
    return 0;
  else if (pending < 0)
    ; /* Feck arse */
  else
  {
    /* Make sure we use up all pending data, otherwise select
     * will wastefully spin */
    while (pending)
    {
      impulse_int used;
      used = _p_impulse_channel_process(chan, pending);
      pending -= used;
    }
  }
  return impulse_queue_count(chan->internal->queue);
}


impulse_int impulse_channel_send(impulse_channel chan,
				 impulse_message out)
{
  impulse_client_write(chan->client, out->data, out->data_length);
  return 0;
}


impulse_int impulse_channel_receive(impulse_channel chan,
				    impulse_message *in)
{
  *in = (impulse_message)impulse_queue_serve(chan->internal->queue);
  return 0;
}


/* End of file */
