/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * client.c
 * --------
 *
 * Communication transport functions
 * Client implementation
 */

#include "../../../config.h"

#include <impulse/comms/client.h>
#include <impulse/util/memory.h>


struct _impulse_client
{
  impulse_transport transport;
  void *data;
};


impulse_client impulse_client_create(impulse_transport transport,
				     void *data)
{
  impulse_client client = impulse_alloc(sizeof(struct _impulse_client));
  
  client->transport = transport;
  client->data = data;
    
  return client;
}


void impulse_client_destroy(impulse_client client)
{
  client->transport->client.destroy(client->data);
  impulse_free(client);
}


impulse_bool impulse_client_is_open(impulse_client client)
{
  return client->transport->client.is_open(client->data);
}


impulse_int impulse_client_open(impulse_client client,
				impulse_unistring address_local,
				impulse_unistring address_remote)
{
  return client->transport->client.open(client->data,
					address_local,
					address_remote);
}


impulse_int impulse_client_close(impulse_client client)
{
  return client->transport->client.close(client->data);
}


impulse_int impulse_client_handle(impulse_client client)
{
  return client->transport->client.handle(client->data);
}


impulse_int impulse_client_pending(impulse_client client)
{
  return client->transport->client.pending(client->data);
}


impulse_int impulse_client_read(impulse_client client,
				impulse_byte *buffer,
				impulse_int size)
{
  return client->transport->client.read(client->data,
					buffer,
					size);
}


impulse_int impulse_client_write(impulse_client client,
				 impulse_byte *buffer,
				 impulse_int size)
{
  return client->transport->client.write(client->data,
					 buffer,
					 size);
}


impulse_int impulse_client_address_local(impulse_client client,
					 impulse_unistring *address)
{
  return client->transport->client.address_local(client->data,
						 address);
}


impulse_int impulse_client_address_remote(impulse_client client,
					  impulse_unistring *address)
{
  return client->transport->client.address_remote(client->data,
						  address);
}


/* End of file */
