/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * message.h
 * ---------
 *
 * Communication transport functions
 */


#ifndef __impulse_comms_message_h
#define __impulse_comms_message_h


#include <impulse/util/setup.h>

struct _impulse_message_header
{
  impulse_uint service;
  impulse_ushort opcode;
  impulse_ushort length;
};

typedef struct _impulse_message_header *impulse_message_header;

struct _impulse_message
{
  impulse_uint service;
  impulse_uint opcode;

  impulse_message_header header;
  impulse_ushort header_length;
    
  impulse_ubyte *body;
  impulse_ushort body_length;
  
  void *data;
  impulse_ushort data_length;
};

typedef struct _impulse_message *impulse_message;


impulse_message impulse_message_create(impulse_uint service,
				       impulse_uint opcode,
				       impulse_ushort length);
void impulse_message_destroy(impulse_message msg);

#if 0
#define impulse_message_swap_16(x) \
(impulse_short) \
((((impulse_short)x) & 0xff00 >> 8) | (((impulse_short)x) & 0x00ff << 8))

#define impulse_message_swap_32(x) \
(impulse_int) \
(((((impulse_int)x) & 0xff000000) >> 24) | ((((impulse_int)x) & 0x00ff0000) >>  8) | \
 ((((impulse_int)x) & 0x0000ff00) <<  8) | ((((impulse_int)x) & 0x000000ff) << 24))

#define impulse_message_swap_64(x) \
(impulse_long) \
(((((impulse_long)x) & 0xff00000000000000) >> 56) | ((((impulse_long)x) & 0x00ff000000000000) >> 40) | \
 ((((impulse_long)x) & 0x0000ff0000000000) >> 24) | ((((impulse_long)x) & 0x000000ff00000000) >>  8) | \
 ((((impulse_long)x) & 0x00000000ff000000) <<  8) | ((((impulse_long)x) & 0x0000000000ff0000) << 24) | \
 ((((impulse_long)x) & 0x000000000000ff00) << 40) | ((((impulse_long)x) & 0x00000000000000ff) << 56))



#define _impulse_message_read(msg, val, type) \
(val) = *((type*)(msg)->offset)++

#define _impulse_message_write(msg, val, type) \
*((type*)(msg)->offset)++ = (val)



#define _impulse_message_read_8(msg, val, type) \
_impulse_message_read(msg, val, type)

#define _impulse_message_read_16(msg, val, type) \
do { \
_impulse_message_read(msg, val, type); \
if (msg->order != IMPULSE_BYTE_ORDER) \
  val = impulse_message_swap_16(val); \
} while(0)

#define _impulse_message_read_32(msg, val, type) \
do { \
_impulse_message_read(msg, val, type); \
if (msg->order != IMPULSE_BYTE_ORDER) \
  val = impulse_message_swap_32(val); \
} while(0)

#define _impulse_message_read_64(msg, val, type) \
do { \
_impulse_message_read(msg, val, type); \
if (msg->order != IMPULSE_BYTE_ORDER) \
  val = impulse_message_swap_64(val); \
} while(0)



#define _impulse_message_write_8(msg, val, type) \
_impulse_message_write(msg, val, type)

#define _impulse_message_write_16(msg, val, type) \
do { \
if (msg->order == IMPULSE_BYTE_ORDER) \
  _impulse_message_write(msg, val, type); \
else \
  _impulse_message_write(msg, impulse_message_swap_16(val), type); \
} while(0)

#define _impulse_message_write_32(msg, val, type) \
do { \
if (msg->order == IMPULSE_BYTE_ORDER) \
  _impulse_message_write(msg, val, type); \
else \
  _impulse_message_write(msg, impulse_message_swap_32(val), type); \
} while(0)

#define _impulse_message_write_64(msg, val, type) \
do { \
if (msg->order == IMPULSE_BYTE_ORDER) \
  _impulse_message_write(msg, val, type); \
else \
  _impulse_message_write(msg, impulse_message_swap_64(val), type); \
} while(0)



#define impulse_message_read_ubyte(msg, val)     _impulse_message_read_8 (msg, val, impulse_ubyte)
#define impulse_message_read_ushort(msg, val)    _impulse_message_read_16(msg, val, impulse_ushort)
#define impulse_message_read_uint(msg, val)      _impulse_message_read_32(msg, val, impulse_uint)
#define impulse_message_read_ulong(msg, val)     _impulse_message_read_64(msg, val, impulse_ulong)

#define impulse_message_read_byte(msg, val)      _impulse_message_read_8 (msg, val, impulse_byte)
#define impulse_message_read_short(msg, val)     _impulse_message_read_16(msg, val, impulse_short)
#define impulse_message_read_int(msg, val)       _impulse_message_read_32(msg, val, impulse_int)
#define impulse_message_read_long(msg, val)      _impulse_message_read_64(msg, val, impulse_long)

#define impulse_message_read_char(msg, val)      _impulse_message_read_8 (msg, val, impulse_char)
#define impulse_message_read_unichar(msg, val)   _impulse_message_read_16(msg, val, impulse_unichar)
#define impulse_message_read_bool(msg, val)      _impulse_message_read_8 (msg, val, impulse_bool)

#define impulse_message_read_float(msg, val) \
do { \
  union \
  { \
    impulse_float real; \
    impulse_int fake; \
  } x; \
  x.real = val; \
  _impulse_message_read_32(msg, x.fake, impulse_int); \
  val = x.real; \
} while(0)
  
#define impulse_message_read_double(msg, val) \
do { \
  union \
  { \
    impulse_double real; \
    impulse_long fake; \
  } x; \
  x.real = val; \
  _impulse_message_read_64(msg, x.fake, impulse_long); \
  val = x.real; \
} while(0)


#define impulse_message_write_ubyte(msg, val)    _impulse_message_write_8 (msg, val, impulse_ubyte)
#define impulse_message_write_ushort(msg, val)   _impulse_message_write_16(msg, val, impulse_ushort)
#define impulse_message_write_uint(msg, val)     _impulse_message_write_32(msg, val, impulse_uint)
#define impulse_message_write_ulong(msg, val)    _impulse_message_write_64(msg, val, impulse_ulong)

#define impulse_message_write_byte(msg, val)     _impulse_message_write_8 (msg, val, impulse_byte)
#define impulse_message_write_short(msg, val)    _impulse_message_write_16(msg, val, impulse_short)
#define impulse_message_write_int(msg, val)      _impulse_message_write_32(msg, val, impulse_int)
#define impulse_message_write_long(msg, val)     _impulse_message_write_64(msg, val, impulse_long)
#define impulse_message_write_char(msg, val)     _impulse_message_write_8 (msg, val, impulse_char)
#define impulse_message_write_unichar(msg, val)  _impulse_message_write_16(msg, val, impulse_unichar)
#define impulse_message_write_bool(msg, val)     _impulse_message_write_8 (msg, val, impulse_bool)

#define impulse_message_write_float(msg, val) \
do { \
  union \
  { \
    impulse_float real; \
    impulse_int fake; \
  } x; \
  x.real = val; \
  _impulse_message_write_32(msg, x.fake, impulse_int); \
} while(0)

#define impulse_message_write_double(msg, val) \
do { \
  union \
  { \
    impulse_double real; \
    impulse_long fake; \
  } x; \
  x.real = val; \
  _impulse_message_write_64(msg, x.fake, impulse_long); \
} while(0)


#endif

#endif

/* End of file */

