/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * peer.c
 * ------
 *
 * Communication transport functions
 * Peer-to-peer implementation
 */

#include "../../../config.h"

#include "impulse/comms/peer.h"
#include "impulse/util/memory.h"

struct _impulse_peer
{
  impulse_transport transport;
  void *data;
};


impulse_peer impulse_peer_create(impulse_transport transport,
				 void *data)
{
  impulse_peer peer = impulse_alloc(sizeof(struct _impulse_peer));
  
  peer->transport = transport;
  peer->data = data;
  
  return peer;
}


void impulse_peer_destroy(impulse_peer peer)
{
  peer->transport->peer.destroy(peer->data);
  
  impulse_free(peer);
}


impulse_bool impulse_peer_is_open(impulse_peer peer)
{
  return peer->transport->peer.is_open(peer->data);
}


impulse_int impulse_peer_open(impulse_peer peer,
			      impulse_unistring address)
{
  return peer->transport->peer.open(peer->data,
				    address);
}


impulse_int impulse_peer_close(impulse_peer peer)
{
  return peer->transport->peer.close(peer->data);
}


impulse_int impulse_peer_handle(impulse_peer peer)
{
  return peer->transport->peer.handle(peer->data);
}


impulse_int impulse_peer_pending(impulse_peer peer)
{
  return peer->transport->peer.pending(peer->data);
}


impulse_int impulse_peer_send(impulse_peer peer,
			      impulse_message message,
			      impulse_unistring address)
{
  return peer->transport->peer.send(peer->data,
				    message,
				    address);
}


impulse_int impulse_peer_receive(impulse_peer peer,
				 impulse_message *message,
				 impulse_unistring *address)
{
  return peer->transport->peer.receive(peer->data,
				       message,
				       address);
}


impulse_int impulse_peer_address(impulse_peer peer,
				 impulse_unistring *address)
{
  return peer->transport->peer.address(peer->data,
				       address);
}


/* End of file */
