/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * server.c
 * --------
 *
 * Communication transport functions
 * Server implementation
 */

#include "../../../config.h"

#include "impulse/comms/server.h"
#include "impulse/util/memory.h"


struct _impulse_server
{
  impulse_transport transport;
  void *data;
};


impulse_server impulse_server_create(impulse_transport transport,
				     void *data)
{
  impulse_server server = impulse_alloc(sizeof(struct _impulse_server));
    
  server->transport = transport;
  server->data = data;
    
  return server;
}


void impulse_server_destroy(impulse_server server)
{
  server->transport->server.destroy(server->data);
  impulse_free(server);
}


impulse_bool impulse_server_is_open(impulse_server server)
{
  return server->transport->server.is_open(server->data);
}


impulse_int impulse_server_open(impulse_server server,
				impulse_unistring address)
{
  return server->transport->server.open(server->data,
					address);
}


impulse_int impulse_server_close(impulse_server server)
{
  return server->transport->server.close(server->data);
}


impulse_int impulse_server_handle(impulse_server server)
{
  return server->transport->server.handle(server->data);
}


impulse_int impulse_server_pending(impulse_server server)
{
  return server->transport->server.pending(server->data);
}


impulse_int impulse_server_accept(impulse_server server,
				  impulse_client *client)
{
  void *impl = NULL;
  impulse_int ret = server->transport->server.accept(server->data,
						     &impl);
  if (impl)
  {
    *client = impulse_client_create(server->transport,
				    impl);
  }
  else
    *client = NULL;
    
  return ret;
}


impulse_int impulse_server_address(impulse_server server,
				   impulse_unistring *address)
{
  return server->transport->server.address(server->data,
					   address);
}


/* End of file */
