/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * transport.h
 * -----------
 *
 * Communication transport protocol
 */


#ifndef __impulse_comms_transport_h
#define __impulse_comms_transport_h

#include <impulse/util/unistring.h>
#include <impulse/comms/message.h>

struct _impulse_transport
{
  impulse_unistring name;
  
  struct 
  {
    void *(*create)(void);
    void (*destroy)(void *);

    impulse_bool (*is_open)(void *);
    impulse_int (*open)(void *,
			impulse_unistring);
    impulse_int (*close)(void *);
    impulse_int (*handle)(void *);

    impulse_int (*pending)(void *);
    impulse_int (*accept)(void *,
			  void **);

    impulse_int (*address)(void *,
			   impulse_unistring *);
  } server;
  
  struct
  {
    void *(*create)(void);
    void (*destroy)(void *);

    impulse_bool (*is_open)(void *);
    impulse_int (*open)(void *,
			impulse_unistring,
			impulse_unistring);
    impulse_int (*close)(void *);
    impulse_int (*handle)(void *);

    impulse_int (*pending)(void *);
    impulse_int (*read)(void *,
			impulse_byte *,
			impulse_int);
    impulse_int (*write)(void *,
			 impulse_byte *,
			 impulse_int);

    impulse_int (*address_local)(void *,
				 impulse_unistring *);
    impulse_int (*address_remote)(void *,
				  impulse_unistring *);
  } client;

  struct 
  {
    void *(*create)(void);
    void (*destroy)(void *);

    impulse_bool (*is_open)(void *);
    impulse_int (*open)(void *,
			impulse_unistring);
    impulse_int (*close)(void *);
    impulse_int (*handle)(void *);

    impulse_int (*pending)(void *);
    impulse_int (*send)(void *,
			impulse_message,
			impulse_unistring);
    impulse_int (*receive)(void *,
			   impulse_message *,
			   impulse_unistring *);

    impulse_int (*address)(void *,
			   impulse_unistring *);
  } peer;
};  

typedef struct _impulse_transport *impulse_transport;

#endif

/* End of file */
