/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * transport_ipc.c
 * ----------------
 *
 * Communication transport functions
 * SysV IPC protocols
 * 
 * Addresses are specified in the form:
 * 
 * protocol:node
 */

#include "../../../config.h"

#include "impulse/comms/transport_ipc.h"
#include "impulse/util/memory.h"
#include "impulse/util/error.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#if HAVE_COMMS_IPC

struct _impulse_transport_ipcp
{
  impulse_int handle;
  
  impulse_char *node;
};

typedef struct _impulse_transport_ipcp *impulse_transport_ipcp;


/*                            ======--------======                           */


/* Peer functions */
void *impulse_transport_ipcp_create(void);
void impulse_transport_ipcp_destroy(void *data);

impulse_bool impulse_transport_ipcp_is_open(void *data);
impulse_int impulse_transport_ipcp_open(void *data,
					 impulse_unistring address);
impulse_int impulse_transport_ipcp_close(void *data);
impulse_int impulse_transport_ipcp_handle(void *data);

impulse_int impulse_transport_ipcp_pending(void *data);
impulse_int impulse_transport_ipcp_send(void *data,
					 impulse_message msg,
					 impulse_unistring address);
impulse_int impulse_transport_ipcp_receive(void *data,
					    impulse_message *msg,
					    impulse_unistring *address);

impulse_int impulse_transport_ipcp_address(void *data,
					    impulse_unistring *address);

/*                            ======--------======                           */


/* Static unicode strings for protocol name */
static struct _impulse_unistring _name_ipc =
{
  3, 3,
  (impulse_unichar[]){
    (impulse_unichar)'i',
    (impulse_unichar)'p',
    (impulse_unichar)'c', 
    (impulse_unichar)'\0'
  }
};

/* Transport function blocks */

static struct _impulse_transport _impulse_transport_ipc =
{
  &_name_ipc,
  {
    NULL, NULL, NULL,
    NULL, NULL, NULL,
    NULL, NULL, NULL
  },
  {
    NULL, NULL, NULL,
    NULL, NULL, NULL,
    NULL, NULL, NULL,
    NULL, NULL
  },
  {
    impulse_transport_ipcp_create,
    impulse_transport_ipcp_destroy,
    impulse_transport_ipcp_is_open,
    impulse_transport_ipcp_open,
    impulse_transport_ipcp_close,
    impulse_transport_ipcp_handle,
    impulse_transport_ipcp_pending,
    impulse_transport_ipcp_send,
    impulse_transport_ipcp_receive,
    impulse_transport_ipcp_address
  }
};

impulse_transport impulse_transport_ipc = &_impulse_transport_ipc;


/*                            ======--------======                           */

/* Peer functions */

void *impulse_transport_ipcp_create(void)
{
  impulse_transport_ipcp peer = impulse_alloc(sizeof(struct _impulse_transport_ipcp));

  peer->handle = -1;
  peer->node = NULL;
  
  return peer;
}


void impulse_transport_ipcp_destroy(void *data)
{
  impulse_transport_ipcp peer = (impulse_transport_ipcp)data;
  
  if (peer->handle)
    impulse_transport_ipcp_close(data);
  
  if (peer->node)
    impulse_free(peer->node);
  impulse_free(peer);
}


impulse_bool impulse_transport_ipcp_is_open(void *data)
{
  impulse_transport_ipcp peer = (impulse_transport_ipcp)data;
  
  return peer->handle != -1;
}


impulse_int impulse_transport_ipcp_open(void *data,
					 impulse_unistring address)
{
  impulse_transport_ipcp peer = (impulse_transport_ipcp)data;
  return 0;
}


impulse_int impulse_transport_ipcp_close(void *data)
{
  impulse_transport_ipcp peer = (impulse_transport_ipcp)data;

  peer->handle = -1;
  impulse_free(peer->node);
  peer->node = NULL;
  
  return 0;
}


impulse_int impulse_transport_ipcp_handle(void *data)
{
  impulse_transport_ipcp peer = (impulse_transport_ipcp)data;
  
  return peer->handle;
}


impulse_int impulse_transport_ipcp_pending(void *data)
{
  impulse_transport_ipcp peer = (impulse_transport_ipcp)data;
}


impulse_int impulse_transport_ipcp_send(void *data,
					 impulse_message msg,
					 impulse_unistring address)
{
  impulse_transport_ipcp peer = (impulse_transport_ipcp)data;

  return -1;
}


impulse_int impulse_transport_ipcp_receive(void *data,
					    impulse_message *msg,
					    impulse_unistring *address)
{
  impulse_transport_ipcp peer = (impulse_transport_ipcp)data;
  return -1;
}


impulse_int impulse_transport_ipcp_address(void *data,
					    impulse_unistring *address)
{
  impulse_transport_ipcp peer = (impulse_transport_ipcp)data;
  return 0;
}

#else

impulse_transport impulse_transport_ipc = NULL;

#endif

/* End of file */
