/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * child_xwin.c
 * ------------
 *
 * Architecture-specific child functions
 */

#include "impulse/util/event.h"
#include "impulse/util/memory.h"
#include "impulse/rows/child_xwin.h"
#include "impulse/rows/main_xwin.h"
#include "impulse/rows/screen.h"

/*                            ======--------======                           */

static impulse_bool window_event(impulse_event *event,
			void *data);

/*                            ======--------======                           */

void _p_ROWS_child_int_setup(ROWS_widget   *widget,
			     ROWS_drawable  drawable)
{
  _p_ROWS_child_data   **private;
  _p_ROWS_X_child_data  *data;

  private = (_p_ROWS_child_data **) ROWS_widget_private(widget,
							_p_ROWS_child_ID);

  data = (*private)->int_data =
    impulse_alloc(sizeof(struct _p_ROWS_X_child_data));

  data->window     = None;
  data->parent     = None;
  data->display    = appl->display;
  data->actualised = 0;
  data->drawable   = NULL;
}

void _p_ROWS_child_int_destroy(_p_ROWS_child_data **data)
{
  _p_ROWS_X_child_data  *x_data;

  x_data = (*data)->int_data;
  if (x_data->window != None)
    {
      XDestroyWindow(x_data->display,
		     x_data->window);
    }
  
  impulse_free((*data)->int_data);
}

void _p_ROWS_child_int_container(ROWS_child     child,
				 ROWS_container container)
{
  _p_ROWS_child_data   **private;
  _p_ROWS_X_child_data  *data;
 
  Window container_win;

  private = (_p_ROWS_child_data **) child;

  data = (*private)->int_data;

  data->parent =
    container_win = ROWS_X_child_get_window(ROWS_child_cast(container));

  if (data->actualised)
    {
      ROWS_child_actual_widget(child);
    }
  else
    {
      data->window = data->parent;
    }
}

/*                            ======--------======                           */

Window ROWS_X_child_get_window(ROWS_child child)
{
  _p_ROWS_child_data   **private;
  _p_ROWS_X_child_data  *x_data;
  Window                 win;
  
  ROWS_child_lock(child);
  ROWS_widget_check_type(child, _p_ROWS_child_type, "ROWS_child");

  private = (_p_ROWS_child_data **) child;
  
  x_data = (*private)->int_data;

  win = x_data->window;

  ROWS_child_unlock(child);

  return win;
}

void ROWS_X_child_set_window(ROWS_child child,
			     Window     win)
{
  _p_ROWS_child_data   **private;
  _p_ROWS_X_child_data  *x_data;
  ROWS_drawable          old_drawable;
  
  ROWS_child_lock(child);
  ROWS_widget_check_type(child, _p_ROWS_child_type, "ROWS_child");

  private = (_p_ROWS_child_data **) child;
  
  x_data = (*private)->int_data;

  if (x_data->window == x_data->parent)
    x_data->parent = win;
  
  x_data->window = win;

  old_drawable = x_data->drawable;
  
  x_data->drawable = ROWS_screen_create(child);
  ROWS_child_register_drawable(child,
			       ROWS_drawable_cast(x_data->drawable));

  if (old_drawable != NULL)
    {
      ROWS_widget_destroy(old_drawable);
    }

  ROWS_child_unlock(child);
}

static void _p_ROWS_X_child_position_window(ROWS_child child,
					    Display   *disp,
					    Window     win)
{
  ROWS_point     *origin;
  ROWS_point     *size;
  XWindowChanges  changes;

  origin = ROWS_child_get_origin(child);
  size   = ROWS_box_get_physical(ROWS_box_cast(child));

  if (size->x <= 0)
    size->x = PIXEL_TO_MICRONS(2);
  if (size->y <= 0)
    size->y = PIXEL_TO_MICRONS(2);

  changes.x      = MICRONS_TO_PIXEL(origin->x);
  changes.y      = MICRONS_TO_PIXEL(origin->y);
  changes.width  = MICRONS_TO_PIXEL(size->x);
  changes.height = MICRONS_TO_PIXEL(size->y);
  
  XConfigureWindow(disp,
		   win,
		   CWX|CWY|CWWidth|CWHeight,
		   &changes);
//  XFlush(
//  XSync(disp, False);
}

void ROWS_X_child_window_attributes(ROWS_child child)
{
  _p_ROWS_child_data   **private;
  _p_ROWS_X_child_data  *x_data;
  XSetWindowAttributes   attributes;
  ROWS_style            style;
 
  ROWS_child_lock(child);
  ROWS_widget_check_type(child, _p_ROWS_child_type, "ROWS_child");

  style = ROWS_child_get_style(child);

  private = (_p_ROWS_child_data **) child;  
  x_data = (*private)->int_data;

  attributes.colormap         = appl->cmap;
  attributes.background_pixel = ROWS_brush_get_fgpixel(style->brush[ROWS_STYLE_NORMAL][ROWS_STYLE_BACKGROUND]);
  attributes.border_pixel     = ROWS_brush_get_fgpixel(style->brush[ROWS_STYLE_NORMAL][ROWS_STYLE_BACKGROUND]);

  XChangeWindowAttributes(x_data->display,
			  x_data->window,
			  CWBackPixel | CWBorderPixel | CWColormap,
			  &attributes);

  XFlush(x_data->display);

  ROWS_child_unlock(child);
}

void ROWS_X_child_create_window(ROWS_child child,
				Window     parent)
{
  _p_ROWS_child_data   **private;
  _p_ROWS_X_child_data  *x_data;
  Window                 win;

  ROWS_child_lock(child);

  ROWS_widget_check_type(child, _p_ROWS_child_type, "ROWS_child");

  private = (_p_ROWS_child_data **) child;
  (*private)->do_redraw = 0;
  x_data = (*private)->int_data;
      
  x_data->window =
    win = XCreateSimpleWindow(x_data->display,
			      x_data->parent,
			      0,0,1,1,
			      0,
			      BlackPixel(x_data->display, 0),
			      WhitePixel(x_data->display, 0));

  impulse_event_register(IMPULSE_EVENT_GUI,
			 win,
			 window_event,
			 child);

  XSelectInput(x_data->display, x_data->window,
	    /* StructureNotifyMask | */
	    /* KeyPressMask        | */
	    /* KeyReleaseMask      | */
	       ButtonPressMask     |
	       ButtonReleaseMask   |
	       EnterWindowMask     |
	       LeaveWindowMask     |
	    /* PointerMotionMask   | */
	       ButtonMotionMask    |
	       ExposureMask        |
	    /* VisibilityChangeMask| */
	    /* ResizeRedirectMask  | */
	    /* FocusChangeMask     | */
	       PropertyChangeMask);

  ROWS_X_child_window_attributes(child);

  ROWS_child_unlock(child);
}

void ROWS_child_actual_widget(ROWS_child child)
{
  _p_ROWS_child_data   **private;
  _p_ROWS_X_child_data  *x_data;
  Window                 win;
  
  ROWS_child_lock(child);

  ROWS_widget_check_type(child, _p_ROWS_child_type, "ROWS_child");

  private = (_p_ROWS_child_data **) child;  
  x_data = (*private)->int_data;

  x_data->actualised = 1;
  if (x_data->parent == None)
    {
      ROWS_child_unlock(child);
      return;
    }

  if (x_data->window == None || x_data->window == x_data->parent)
    {
      ROWS_drawable old_drawable;
      old_drawable = x_data->drawable;
      ROWS_X_child_create_window(child,
				 x_data->parent);
      win = x_data->window;
      x_data->drawable = ROWS_screen_create(child);
      ROWS_child_register_drawable(child,
      				   ROWS_drawable_cast(x_data->drawable));
      if (old_drawable != NULL)
	{
	  ROWS_widget_destroy(old_drawable);
	}

      if ((*private)->shown)
	XMapWindow(x_data->display, win);
    }
  else
    {
      win = x_data->window;
      XUnmapWindow(x_data->display, win);
      XReparentWindow(x_data->display,
		      win,
		      x_data->parent,
		      0,0);

      if ((*private)->shown)
	XMapWindow(x_data->display, win);
    }

  _p_ROWS_X_child_position_window(child, x_data->display, win);

  ROWS_child_unlock(child);
}

void _p_ROWS_child_int_resize(ROWS_child child)
{
  _p_ROWS_child_data   **private;
  _p_ROWS_X_child_data  *x_data;
  static ROWS_point      zero = {0,0};

  private = (_p_ROWS_child_data **) child;  
  x_data = (*private)->int_data;

  if (x_data->window != None && x_data->window != x_data->parent)
    {
      _p_ROWS_X_child_position_window(child, x_data->display,
				      x_data->window);
      
      ROWS_child_set_origin(child, &zero);
    }
}

void _p_ROWS_child_int_update(ROWS_child child)
{
  _p_ROWS_child_data     **private;
  _p_ROWS_X_child_data    *x_data;
  ROWS_style_active_type   type;
  ROWS_style              style;
  XSetWindowAttributes     attributes;  

  private = (_p_ROWS_child_data **) child;  
  x_data  = (*private)->int_data;

  type    = ROWS_child_get_activation(child);
  style   = ROWS_child_get_style(child);

  if (x_data->actualised)
    {
      if (x_data->window != None && x_data->window != x_data->parent)
	{
	  attributes.background_pixel =
	    ROWS_brush_get_fgpixel(style->brush[type][ROWS_STYLE_BACKGROUND]);
	  attributes.border_pixel     =
	    ROWS_brush_get_fgpixel(style->brush[type][ROWS_STYLE_BACKGROUND]);

	  XChangeWindowAttributes(x_data->display,
				  x_data->window,
				  CWBackPixel | CWBorderPixel,
				  &attributes);
	  XClearArea(x_data->display,
		     x_data->window,
		     0,0,0,0,
		     True);
	}
    }
}

void _p_ROWS_child_int_show(ROWS_child child)
{
  _p_ROWS_child_data   **private;
  _p_ROWS_X_child_data  *x_data;

  private = (_p_ROWS_child_data **) child;  
  x_data = (*private)->int_data;

  if (x_data->actualised)
    {
      if (x_data->window != None && x_data->window != x_data->parent)
	{
	  XMapWindow(x_data->display, x_data->window);
	  _p_ROWS_X_child_position_window(child,
					  x_data->display,
					  x_data->window);
	}
    }
}

void _p_ROWS_child_int_hide(ROWS_child child)
{
  _p_ROWS_child_data   **private;
  _p_ROWS_X_child_data  *x_data;

  private = (_p_ROWS_child_data **) child;  
  x_data = (*private)->int_data;

  if (x_data->actualised)
    {
      if (x_data->window != None && x_data->window != x_data->parent)
	{
	  XUnmapWindow(x_data->display, x_data->window);
	}
    }
}

void _p_ROWS_child_int_realised(ROWS_child child)
{
  _p_ROWS_child_data   **private;
  _p_ROWS_X_child_data  *x_data;

  private = (_p_ROWS_child_data **) child;  
  x_data = (*private)->int_data;

  if (x_data->actualised)
    {
      ROWS_child_actual_widget(child);
    }

  if (x_data->window != None && x_data->window != x_data->parent)
    _p_ROWS_X_child_position_window(child, x_data->display, x_data->window);
}

/*                            ======--------======                           */

static int _p_child_expose_pred(impulse_event *event,
				 void          *data)
{
  _p_ROWS_X_child_data *x_data;

  if (event->type != IMPULSE_EVENT_GUI)
    return 0;
  
  x_data = data;

  if (event->data.gui.object == x_data->window &&
      event->data.gui.type == Expose)
      return 1;
  else
    return 0;
}

static void _p_child_expose(XEvent     *ev,
			    ROWS_child  child)
{
  _p_ROWS_child_data   **private;
  _p_ROWS_X_child_data   *x_data;
  ROWS_child_event        event; 
  ROWS_drawable           drawable;
  
  private = (_p_ROWS_child_data **) child;
  x_data  = (*private)->int_data;

  event.event.redraw.invalid.pos.x  = PIXEL_TO_MICRONS(ev->xexpose.x);
  event.event.redraw.invalid.pos.y  = PIXEL_TO_MICRONS(ev->xexpose.y);
  event.event.redraw.invalid.size.x = PIXEL_TO_MICRONS(ev->xexpose.width);
  event.event.redraw.invalid.size.y = PIXEL_TO_MICRONS(ev->xexpose.height);

  ROWS_box_get_boundaries(ROWS_box_cast(child),
			  &event.event.redraw.clip);  

  drawable = ROWS_child_get_drawable(ROWS_child_cast(child));
  ROWS_drawable_clip_push(drawable);

  if (ev->xexpose.count)
    {
      impulse_event im_event;
      ROWS_rect rect;
      
      ROWS_drawable_clip_region(drawable, 1);
      ROWS_drawable_clip_region_clear(drawable);
      
      ROWS_drawable_clip_region_add(drawable,
				    &event.event.redraw.invalid);
      
      while (impulse_event_check_next(&im_event,
				      _p_child_expose_pred,
				      private))
	{
	  
	
	ev = im_event.data.gui.data;
	
	rect.pos.x  = ev->xexpose.x;
	rect.pos.y  = ev->xexpose.y;
	rect.size.x = ev->xexpose.width;
	rect.size.y = ev->xexpose.height;
	
	ROWS_drawable_clip_region_add(drawable,
				      &rect);

//	done = ev->xexpose.count == 0;
	impulse_free(ev);
      }

      ROWS_drawable_clip_region_set(drawable);
      ROWS_drawable_clip_region_bounds(drawable, 
					   &event.event.redraw.invalid);
    }
  else
    {
      ROWS_drawable_clip_region(drawable, 0);
      ROWS_drawable_set_clip(drawable,
			     &event.event.redraw.invalid);
    }

  ROWS_child_event_send(child,
			ROWS_CHILD_REDRAW,
			&event);

  XFlush(x_data->display);

  ROWS_drawable_clip_pop(drawable);
}


static impulse_bool window_event(impulse_event  *event,
			void *data)
{
  XEvent              *ev;
  ROWS_child widget = data;
  
  ev = event->data.gui.data;
  
  ROWS_child_lock(widget);

  switch (ev->type)
    {
    case ConfigureNotify:
      /*
       * We know about these, but child configuration is actually done 
       * elsewhere
       */
      break;

    case Expose:
      /*
       * Generate redraw events
       */
      _p_child_expose(ev,
		      widget);
      break;

    case ButtonPress:
    case ButtonRelease:
      /*
       * The user has done something with the mouse
       */
      {
	ROWS_child_event event;

	event.event.click.Xoffset  = ev->xbutton.x;
	event.event.click.Yoffset  = ev->xbutton.y;
	event.event.click.button   = ev->xbutton.button;
	event.event.click.released = (ev->xbutton.type ==
				      ButtonRelease);

	ROWS_child_event_send(ROWS_child_cast(widget),
			      ROWS_CHILD_CLICK,
			      &event);
      }
      break;

    case EnterNotify:
      {
	ROWS_child_event event;

	event.event.enter.x = ev->xcrossing.x;
	event.event.enter.y = ev->xcrossing.y;
	event.event.enter.state =
	  ROWS_X_get_button_state(ev->xcrossing.state);

	ROWS_child_event_send(ROWS_child_cast(widget),
			      ROWS_CHILD_POINTER_ENTER,
			      &event);	
      }
      break;

    case LeaveNotify:
      {
	ROWS_child_event event;

	event.event.enter.x = ev->xcrossing.x;
	event.event.enter.y = ev->xcrossing.y;
	event.event.enter.state =
	  ROWS_X_get_button_state(ev->xcrossing.state);

	ROWS_child_event_send(ROWS_child_cast(widget),
			      ROWS_CHILD_POINTER_LEAVE,
			      &event);	
      }
      break;

    case MotionNotify:
      {
	ROWS_child_event event;

	event.event.enter.x = ev->xmotion.x;
	event.event.enter.y = ev->xmotion.y;
	event.event.enter.state =
	  ROWS_X_get_button_state(ev->xmotion.state);

	ROWS_child_event_send(ROWS_child_cast(widget),
			      ROWS_CHILD_POINTER_MOVE,
			      &event);	
      }
      break;
      
    default:
      break;
    }

  ROWS_child_unlock(widget);
  impulse_free(ev);

  return 1;
}
