/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * container.h
 * -----------
 *
 * Something that contains a child
 */

#ifndef __impulse_rows_container_h
#define __impulse_rows_container_h

#include <impulse/rows/child.h>

/*                            ======--------======                           */

/* Container widget type */

typedef void **ROWS_container;

/* Contained widget type */

/*                            ======--------======                           */

/* Events */

typedef enum {
  ROWS_CONTAINER_CHANGED = 0,
  ROWS_CONTAINER_SIZE,

  ROWS_CONTAINER_MAX_EVENT
} ROWS_container_event_num;

struct ROWS_child_changed {
  ROWS_child changed;
};

struct ROWS_child_size {
  ROWS_child to_size;
};

struct _ROWS_container_event {
  union {
    struct ROWS_child_changed changed;
    struct ROWS_child_size    size;
  } event;
};

typedef struct _ROWS_container_event ROWS_container_event;

/* Functions */

typedef int (*ROWS_container_func) (ROWS_container            widget,
				    ROWS_container_event_num  event_number,
				    ROWS_container_event     *event,
				    void                     *data);

/*                            ======--------======                           */

/* Standard widget functions */

extern ROWS_widget            ROWS_container_create         (ROWS_drawable drawable);
extern ROWS_widget           *ROWS_container_initialise     (int id,
							     ROWS_drawable drawable);
extern inline int             ROWS_container_next_identifier(void);
extern inline ROWS_container  ROWS_container_cast           (void **widget);

/*                            ======--------======                           */

/* Class-specific functions */

extern void ROWS_container_register_event(ROWS_container            child,
					  ROWS_container_event_num  event,
					  ROWS_container_func       func,
					  void                     *data);
extern void ROWS_container_event_send    (ROWS_container            child,
					  ROWS_container_event_num  event,
					  ROWS_container_event     *data);

/* Flags for ROWS_container_add */

#define ROWS_CONTAINER_FILL         1
#define ROWS_CONTAINER_EXPAND       2
#define ROWS_CONTAINER_RIGHTJUSTIFY 4

extern void ROWS_container_changed(ROWS_container container,
				   ROWS_child child);
extern void ROWS_container_size   (ROWS_container container,
				   ROWS_child child);
extern void ROWS_container_add    (ROWS_container container,
				   ROWS_child     child,
				   int            padding,
				   int            flags);

/* Derived widget utility functions */

typedef void (*ROWS_container_iter)(ROWS_container  container,
				    ROWS_child      child,
				    void           *data,
				    void           *moredata);

extern inline void  ROWS_container_add_special(ROWS_container       container,
					       ROWS_child           child,
					       int                  padding,
					       int                  flags,
					       void                *data);
extern inline void  ROWS_container_iterator   (ROWS_container       container,
					       ROWS_container_iter  function,
					       void                *moredata);
extern void        *ROWS_container_lookup     (ROWS_container       container,
					       ROWS_child           child);

#endif

