/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * eventbox.h
 * ----------
 *
 * A box to which events may occur (needs a seperate event delivery system,
 * though)
 */

#ifndef __impulse_rows_eventbox_h
#define __impulse_rows_eventbox_h

#include <impulse/rows/box.h>

/*                            ======--------======                           */

/* EventBox widget type */

typedef void **ROWS_eventbox;

/*                            ======--------======                           */

/* Standard widget functions */

extern ROWS_widget          ROWS_eventbox_create         (void);
extern ROWS_widget         *ROWS_eventbox_initialise     (int id);
extern inline int           ROWS_eventbox_next_identifier(void);
extern inline ROWS_eventbox ROWS_eventbox_cast           (void **widget);

/*                            ======--------======                           */

/* The generic ROWS events */

typedef enum {
  ROWS_event_IDLE   = 0,
  ROWS_event_RESIZE    ,
  ROWS_event_REDRAW    ,
  ROWS_event_SCROLL    ,
  ROWS_event_KEY       ,
  ROWS_event_MOUSECLICK,
  ROWS_event_MOUSEMOVE ,
  ROWS_event_MESSAGE   ,

  ROWS_event_MAXIMUM
} ROWS_event_number;

/* The event typedefs */

struct ROWS_event_idle {
  int null;
};

struct ROWS_event_resize {
  ROWS_point newsize;
};

struct ROWS_event_redraw {
  ROWS_rect clip;
  ROWS_rect invalid;
};

struct ROWS_event_scroll {
  microns oldscrollx, oldscrolly;
  microns newscrollx, newscrolly;
};

struct ROWS_event_key {
  int keycode;
  int state;
  int ascii;
};

struct ROWS_event_mouseclick {
  microns Xoffset, Yoffset;
  int     button;
  int     released;
};

struct ROWS_event_mousemove {
  microns Xoffset, Yoffset;
};

struct ROWS_event_message {
  int   ROWS_message_number;
  char *message_name;
  void *message_data;
};

typedef int (*ROWS_eventbox_func) (void **widget,
				   int    event_num,
				   void  *event_data,
				   void  *data);

/*                            ======--------======                           */

/* Class specific functions */

/*
extern void ROWS_eventbox_general_handler (ROWS_eventbox      box,
					   ROWS_eventbox_func function);
extern void ROWS_eventbox_handler         (ROWS_eventbox     box,
					   ROWS_event_number    event,
					   ROWS_eventbox_func   function);
extern int  ROWS_eventbox_generate        (ROWS_eventbox  box,
					   ROWS_event    *event,
					   void          *data);
*/
					   
extern void        ROWS_eventbox_register_widget   (ROWS_eventbox box,
						    int widget_level,
						    int max_event);
extern void        ROWS_eventbox_register_handler  (ROWS_eventbox box,
						    int widget_level,
						    ROWS_eventbox_func func,
						    int  event_number,
						    void *data);
extern void        ROWS_eventbox_deregister_handler(ROWS_eventbox box,
						    int widget_level,
						    ROWS_eventbox_func func,
						    int  event_number,
						    void *data);
extern inline void ROWS_eventbox_send              (ROWS_eventbox box,
						    int   widget_level,
						    void **widget,
						    int    event_number,
						    void  *event_data);

#endif
