/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the ROWS widget library
 * for Impulse.
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * main_xwin.h
 * -----------
 *
 * ROWS X-specific routines
 * Only include this file in places which are not architechture independant
 * Eventually the routines here will move to a seperate library.
 *
 * Routines here should only be accessed through the ROWS abstraction layer in
 * user programs.
 */

#ifndef __impulse_rows_main_xwin_h
#define __impulse_rows_main_xwin_h

#include <impulse/rows/main.h>
#include <impulse/util/sparse.h>
#include <impulse/util/hash.h>
#include <impulse/util/thread.h>

#include <X11/Xlib.h>

/*                            ======--------======                           */

/* X-reliant definitions */

struct _ROWS_X_application {
  Display     *display;
  Visual      *visual;
  Colormap     cmap;
  Window       screen_window; /* Never displayed, generally the root window */
  impulse_int connection_fd; /* The filedescriptor of the X connection */

  impulse_sparse events;
  impulse_hash fonts;
  impulse_hash atoms;
  
#if USE_THREADS
  impulse_mutex events_lock;
  impulse_mutex fonts_lock;
  impulse_mutex atoms_lock;
  impulse_mutex colourmap_lock;
# if GUI_XWIN_THREAD_PROTECT
  impulse_mutex xlib_lock;
# endif
#endif
  
  
};

typedef struct _ROWS_X_application *ROWS_X_application;

extern ROWS_X_application appl;


#if USE_THREADS && GUI_XWIN_THREAD_PROTECT
extern impulse_mutex_static X_lock;

#define impulse_mutex_X_lock impulse_mutex_lock(appl->xlib_lock);
#define impulse_mutex_X_unlock impulse_mutex_unlock(appl->xlib_lock);
#else
#define impulse_mutex_X_lock
#define impulse_mutex_X_unlock
#endif


Atom ROWS_X_atom_id(impulse_unistring name);


/* Error handling */

extern void ROWS_X_expecting_error(impulse_bool flag);
extern impulse_bool  ROWS_X_get_error_flag (void);

/* Functions to deal with gestures */
extern impulse_int ROWS_X_get_button_state(impulse_int x_state);

#endif
