/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * widget.h
 * --------
 *
 * Base widget class
 */

#ifndef __impulse_rows_widget_h
#define __impulse_rows_widget_h

/*                            ======--------======                           */

/* Base widget type */

typedef void **ROWS_widget;

/* Destruction function */

typedef void (*ROWS_widget_destroy_func) (void **private);
typedef void (*ROWS_widget_destroy_notify_func) (ROWS_widget  destroyed,
						 void        *data);

/*                            ======--------======                           */

/* Creation functions */
extern ROWS_widget  ROWS_widget_create         (void);

/* Destruction functions */
extern inline void         ROWS_widget_destroy        (void **widget);

/* Misc functions */
extern inline ROWS_widget  ROWS_widget_cast           (void **widget);

/* Functions for derived widgets */
extern inline int    ROWS_widget_next_type      (void);
extern inline int    ROWS_widget_next_identifier(void);
extern ROWS_widget  *ROWS_widget_initialise     (int number);
extern void        **ROWS_widget_private        (ROWS_widget *widget,
						 int number);
extern inline void **ROWS_widget_cast_misc      (void **widget,
						 int number,
						 int id);
extern inline void   ROWS_widget_set_type       (ROWS_widget *us,
						 int number,
						 int type_id);
extern inline void   ROWS_widget_set_destructor (ROWS_widget *us,
						 int number,
						 ROWS_widget_destroy_func d);
extern inline void   ROWS_widget_destroy_notify (ROWS_widget us,
						 ROWS_widget_destroy_notify_func f,
						 void *d);
extern inline void   ROWS_widget_check_type     (ROWS_widget widget,
						 int type,
						 char *description);
#if USE_THREADS
extern void ROWS_widget_lock  (ROWS_widget widget);
extern void ROWS_widget_unlock(ROWS_widget widget);
#else
#define ROWS_widget_lock(x)
#define ROWS_widget_unlock(x)
#endif

#endif
