/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * window_xwin.c
 * -------------
 *
 * Implementation of windows for X
 */

#include "impulse/rows/window_xwin.h"
#include "impulse/rows/main_xwin.h"
#include "impulse/rows/child_xwin.h"
#include "impulse/util/event.h"

/*                            ======--------======                           */

extern int _p_ROWS_window_type;
static void *_p_ROWS_window_thread(void *data);

/*                            ======--------======                           */

static int  _p_ROWS_window_expose   (XEvent  *event,
				     ROWS_widget widget);
static int  _p_ROWS_window_map      (XEvent  *event,
				     ROWS_widget widget);
static int  _p_ROWS_window_unmap    (XEvent  *event,
				     ROWS_widget widget);
static int  _p_ROWS_window_configure(XEvent  *event,
				     ROWS_widget widget);
static int  _p_ROWS_window_key      (XEvent  *event,
				     ROWS_widget widget);
static int  _p_ROWS_window_click    (XEvent  *event,
				     ROWS_widget widget);
static int  _p_ROWS_window_client   (XEvent  *event,
				     ROWS_widget widget);
static int  _p_ROWS_window_gainfocus(XEvent  *event,
				     ROWS_widget widget);
static int  _p_ROWS_window_losefocus(XEvent  *event,
				     ROWS_widget widget);

/*                            ======--------======                           */

static int _p_ROWS_window_set_size(ROWS_child            widget,
				   ROWS_child_event_num  event_number,
				   ROWS_child_event     *event,
				   void                 *data);

/*                            ======--------======                           */

/* Class-specific functions */

void ROWS_window_set_title(ROWS_window  window,
			   char        *title)
{
  _p_ROWS_window_data **private;

  ROWS_widget_check_type(window, _p_ROWS_window_type, "ROWS_window");

  ROWS_widget_lock(window);
  private = (_p_ROWS_window_data **) window;

  impulse_free((*private)->properties.title);
  if (title==NULL)
    title = "";

  (*private)->properties.title = impulse_alloc(strlen(title)+1);
  strcpy((*private)->properties.title, title);

  ROWS_widget_unlock(window);
}

void ROWS_window_set_class(ROWS_window  window,
			   char        *class)
{
  _p_ROWS_window_data **private;

  ROWS_widget_check_type(window, _p_ROWS_window_type, "ROWS_window");

  ROWS_widget_lock(window);
  private = (_p_ROWS_window_data **) window;

  impulse_free((*private)->properties.class);
  if (class==NULL)
    class = "";

  (*private)->properties.class = impulse_alloc(strlen(class)+1);
  strcpy((*private)->properties.class, class);

  ROWS_widget_unlock(window);
}

void ROWS_window_set_icon_name(ROWS_window  window,
			       char        *icon_name)
{
  _p_ROWS_window_data **private;

  ROWS_widget_check_type(window, _p_ROWS_window_type, "ROWS_window");

  ROWS_widget_lock(window);
  private = (_p_ROWS_window_data **) window;

  impulse_free((*private)->properties.icon_name);
  if (icon_name==NULL)
    icon_name = "";

  (*private)->properties.icon_name = impulse_alloc(strlen(icon_name)+1);
  strcpy((*private)->properties.icon_name, icon_name);

  ROWS_widget_unlock(window);
}

/*                            ======--------======                           */

/* Event functions */

void ROWS_window_int_setup(ROWS_widget          *widget,
			   _p_ROWS_window_data  *data)
{
  _p_ROWS_X_window_data x_data;
  data->int_data = impulse_alloc(sizeof(struct __p_ROWS_X_window_data));
  x_data = data->int_data;
  
  x_data->created = 0;

  if (x_data->created == 0)
    {
      x_data->created = 1;

      x_data->display = appl->display;
      x_data->window  =
	XCreateSimpleWindow(x_data->display,
			    DefaultRootWindow(x_data->display),
			    0,   0,
			    100, 100,
			    1,
			    BlackPixel(x_data->display, 0),
			    WhitePixel(x_data->display, 0));
      ROWS_X_child_set_window(ROWS_child_cast(*widget),
			      x_data->window);
    }

  ROWS_child_register_event(ROWS_child_cast(*widget),
			    ROWS_CHILD_SET_SIZE,
			    _p_ROWS_window_set_size,
			    NULL);
}

void ROWS_window_int_destroy(_p_ROWS_window_data **private)
{
}


static impulse_bool window_event(impulse_event  *event,
				 void           *data)
{
  XEvent *xevent;
  ROWS_window window = data;
  _p_ROWS_window_data **priv;

  xevent = event->data.gui.data;
  priv = (_p_ROWS_window_data **) data;
  
  /*
   * You may think that I need to call ROWS_child_lock here. Well, you'd
   * be wrong ;-). The container of a window is always NULL, so it is
   * always the topmost widget, so ROWS_child_lock would just lock
   * it and nothing else.
   */
  ROWS_widget_lock((void **) priv);
  
  switch (xevent->type)
  {
   case MapNotify:
    _p_ROWS_window_map(xevent, window);
    break;
      
   case UnmapNotify:
    _p_ROWS_window_unmap(xevent, window);
    break;
  
   case Expose:
    _p_ROWS_window_expose(xevent, window);
    break;

   case ConfigureNotify:
    _p_ROWS_window_configure(xevent, window);
    break;

   case KeyRelease:
    _p_ROWS_window_key(xevent, window);
    break;
  
   case ButtonPress:
   case ButtonRelease:
    _p_ROWS_window_click(xevent, window);
    break;

   case ClientMessage:
    _p_ROWS_window_client(xevent, window);
  }
  
  ROWS_widget_unlock((void **) priv);
  impulse_free(xevent);

  return 1;
}

/*                            ======--------======                           */
     
int  _p_ROWS_window_realise(ROWS_child            widget,
			    ROWS_child_event_num  event_number,
			    ROWS_child_event     *event,
			    void                 *data)
{
  ROWS_window win;
  _p_ROWS_window_data  **private;
  _p_ROWS_X_window_data x_data;
  XSetWindowAttributes   attributes;
  ROWS_style            style;
  Atom                   protocols[2];
  XTextProperty          textproperty;
  XClassHint            *classhint;

  win = ROWS_window_cast(widget);
  private = (_p_ROWS_window_data **)win;
  x_data = (_p_ROWS_X_window_data)(*private)->int_data;

  /* Set our window up properly */
  style = ROWS_child_get_style(widget);

  if (x_data->created == 0)
    {
      ROWS_rect box;

      ROWS_box_get_boundaries(ROWS_box_cast(widget), &box);
      
      x_data->created = 1;

      x_data->display = appl->display;
      x_data->window  =
	XCreateSimpleWindow(x_data->display,
			    DefaultRootWindow(x_data->display),
			    MICRONS_TO_PIXEL(box.pos.x),
			    MICRONS_TO_PIXEL(box.pos.y),
			    MICRONS_TO_PIXEL(box.size.x),
			    MICRONS_TO_PIXEL(box.size.y),
			    1,
			    BlackPixel(x_data->display, 0),
			    WhitePixel(x_data->display, 0));
      ROWS_X_child_set_window(ROWS_child_cast(widget),
			      x_data->window);
      /* ROWS_child_register_drawable(widget,
				   ROWS_screen_create(widget)); */
    }
  
  impulse_mutex_X_lock;
  XSelectInput(x_data->display, x_data->window,
	       StructureNotifyMask |
	       KeyPressMask        |
	       KeyReleaseMask      |
	       ButtonPressMask     |
	       ButtonReleaseMask   |
	       EnterWindowMask     |
	       LeaveWindowMask     |
	    /* PointerMotionMask   | */
	       ButtonMotionMask    |
	       ExposureMask        |
	       VisibilityChangeMask|
	    /* ResizeRedirectMask  | */
	       FocusChangeMask     |
	       PropertyChangeMask);

  attributes.colormap         = appl->cmap;
  attributes.background_pixel = ROWS_brush_get_fgpixel(style->brush[ROWS_STYLE_NORMAL][ROWS_STYLE_BACKGROUND]);
  attributes.border_pixel     = ROWS_brush_get_fgpixel(style->brush[ROWS_STYLE_NORMAL][ROWS_STYLE_BACKGROUND]);

  XChangeWindowAttributes(x_data->display,
			  x_data->window,
			  CWBackPixel | CWBorderPixel | CWColormap,
			  &attributes);

  /* Set window properties */

  /* Titles, icon names, etc */
  XStringListToTextProperty(&(*private)->properties.title, 1, &textproperty);
  XSetWMName(x_data->display,
	     x_data->window,
	     &textproperty);
  XFree(textproperty.value);

  XStringListToTextProperty(&(*private)->properties.icon_name,
			    1, &textproperty);
  XSetWMIconName(x_data->display,
		 x_data->window,
		 &textproperty);
  XFree(textproperty.value);

  /* FIXME: Check for NULL values */
  classhint = XAllocClassHint();
//  classhint->res_name = impulse_alloc(strlen(appl->name)+1);
  classhint->res_class = impulse_alloc(strlen((*private)->properties.class)+1);

//  strcpy(classhint->res_name,  appl->name);
  classhint->res_name = "ROWS Application";
  strcpy(classhint->res_class, (*private)->properties.class);
  XSetClassHint(x_data->display, x_data->window, classhint);

//  impulse_free(classhint->res_name);
  impulse_free(classhint->res_class);
  XFree(classhint);

  /* Protocols */
  protocols[0] = XInternAtom(x_data->display,
			     "WM_DELETE_WINDOW",
			     1);
  /* protocols[1] = XInternAtom((*private)->display,
			     "WM_TAKE_FOCUS",
			     1); */
  XSetWMProtocols(x_data->display,
		  x_data->window,
		  protocols,
		  1);
  impulse_mutex_X_unlock;

#if USE_THREADS
  if ((*private)->enhanced)
    (*private)->thread     = impulse_thread_create_detach(_p_ROWS_window_thread,
							  private);
  else
#endif
    impulse_event_register(IMPULSE_EVENT_GUI,
			   x_data->window,
			   window_event,
			   win);

  return 0;
}

int  _p_ROWS_window_set_size(ROWS_child            widget,
			     ROWS_child_event_num  event_number,
			     ROWS_child_event     *event,
			     void                 *data)
{
  _p_ROWS_window_data  **private;
  _p_ROWS_X_window_data x_data;

  private = (_p_ROWS_window_data **) ROWS_window_cast(widget);
  x_data = (_p_ROWS_X_window_data)(*private)->int_data;

  if (x_data->created)
    {
      XWindowChanges changes;
      ROWS_rect      box;

      ROWS_box_get_boundaries(ROWS_box_cast(widget), &box);

      /*
      changes.x      = box.pos.x;
      changes.y      = box.pos.y;
      */
      changes.width  = MICRONS_TO_PIXEL(box.size.x);
      changes.height = MICRONS_TO_PIXEL(box.size.y);
      
      XConfigureWindow(x_data->display,
		       x_data->window,
		       CWWidth|CWHeight,
		       &changes);
    }
  
  return 0;
}

int  _p_ROWS_window_show(ROWS_child            widget,
			 ROWS_child_event_num  event_number,
			 ROWS_child_event     *event,
			 void                     *data)
{
  _p_ROWS_window_data **private;
  _p_ROWS_X_window_data x_data;

  private = (_p_ROWS_window_data **) ROWS_window_cast(widget);
  x_data = (_p_ROWS_X_window_data)(*private)->int_data;

  /* Show window */
  impulse_mutex_X_lock;
  XMapWindow(x_data->display, x_data->window);
  impulse_mutex_X_unlock;

  return 0;
}

/*                            ======--------======                           */

/* X Event functions */

static int _p_window_configure_pred(impulse_event *event,
				    void          *data)
{
  _p_ROWS_window_data  **private;
  _p_ROWS_X_window_data x_data;

  if (event->type != IMPULSE_EVENT_GUI)
    {
      return 0;
    }
  
  private = data;
  x_data = (_p_ROWS_X_window_data)(*private)->int_data;

  if (event->data.gui.object == x_data->window &&
      event->data.gui.type == ConfigureNotify)
      return 1;
  else
    {
      return 0;
    }
}

static int _p_window_expose_pred(impulse_event *event,
				 void          *data)
{
  _p_ROWS_window_data  **private;
  _p_ROWS_X_window_data x_data;

  if (event->type != IMPULSE_EVENT_GUI)
    return 0;
  
  private = data;
  x_data = (_p_ROWS_X_window_data)(*private)->int_data;

  if (event->data.gui.object == x_data->window &&
      event->data.gui.type == Expose)
      return 1;
  else
    return 0;
}

static int  _p_ROWS_window_expose(XEvent  *xevent,
				  ROWS_widget widget)
{
  _p_ROWS_window_data  **private;
  _p_ROWS_X_window_data x_data;
  ROWS_child_event       event; 
  ROWS_drawable          drawable;
  
  private = (_p_ROWS_window_data **) ROWS_window_cast(widget);
  x_data = (_p_ROWS_X_window_data)(*private)->int_data;

  event.event.redraw.invalid.pos.x  = PIXEL_TO_MICRONS(xevent->xexpose.x);
  event.event.redraw.invalid.pos.y  = PIXEL_TO_MICRONS(xevent->xexpose.y);
  event.event.redraw.invalid.size.x = PIXEL_TO_MICRONS(xevent->xexpose.width);
  event.event.redraw.invalid.size.y = PIXEL_TO_MICRONS(xevent->xexpose.height);

  ROWS_box_get_boundaries(ROWS_box_cast(widget),
			  &event.event.redraw.clip);

  drawable = ROWS_child_get_drawable(ROWS_child_cast(widget));

  ROWS_drawable_clip_push(drawable);
  
  /*
   * The X Protocol says that all Expose events generated by a heirarchy
   * change, occur after the heirarchy change event has occurred. 
   * Thus any subsequent configurenotify events following this cannot be
   * related to this event, and thus must have thier own Expose event even
   * later in the queue. For more details check the XProtocol X11, R6.3
   * page 76.
   * 
   * If planning to re-enable this code for whatever reason, please note
   * the the call to ROWS_X_event_occured will result in deadlock in
   * threaded apps, as the mutex protecting the events register is not
   * recursive. We could use recursive mutexes, but we want to avoid the
   * extra overhead in the event handling.
   * 
   * -- DPB. (29/11/1999)
   */

  /*
   * If there is a configurenotify event, process it first
   */
  #if 0
  ROWS_X_event_flush();

  while (impulse_event_check_next(&im_event,
				  _p_window_configure_pred,
				  private))
    {
      ev = im_event.data.gui.data;
      ROWS_X_event_occured(ev, ev->xany.window);
      impulse_free(ev);
    }
#endif
  
  /* 
   * If there are many pending expose events, try to deal with them all
   * at once
   */
  /*
   * Actaully we can just check that the ev->xexpose.count field is not
   * zero, rather than doing a check event.
   * -- DPB. (29/11/1999)
   */

  if (xevent->xexpose.count)
    {
      impulse_event im_event;
      XEvent *ev;
      ROWS_rect rect;
      impulse_bool done = 0;
      
      ROWS_drawable_clip_region(drawable, 1);
      ROWS_drawable_clip_region_clear(drawable);
      
      ROWS_drawable_clip_region_add(drawable,
				    &event.event.redraw.invalid);

      while(impulse_event_check_next(&im_event,
				     _p_window_expose_pred,
				     private))
      {
	ev = im_event.data.gui.data;
	
	rect.pos.x  = PIXEL_TO_MICRONS(ev->xexpose.x);
	rect.pos.y  = PIXEL_TO_MICRONS(ev->xexpose.y);
	rect.size.x = PIXEL_TO_MICRONS(ev->xexpose.width);
	rect.size.y = PIXEL_TO_MICRONS(ev->xexpose.height);
	
	ROWS_drawable_clip_region_add(drawable,
				      &rect);
	  
	done = ev->xexpose.count == 0;
	impulse_free(ev);
      }

      ROWS_drawable_clip_region_set(drawable);
      ROWS_drawable_clip_region_bounds(drawable, 
					   &event.event.redraw.invalid);
    }
  else
    {
	  ROWS_drawable_clip_region(drawable, 0);
	  ROWS_drawable_set_clip(drawable,
				 &event.event.redraw.invalid);
    }

  ROWS_child_event_send(ROWS_child_cast(widget),
			ROWS_CHILD_REDRAW,
			&event);

  XFlush(x_data->display);

  ROWS_drawable_clip_pop(drawable);
  
  return 1;
}

static int  _p_ROWS_window_map   (XEvent  *event,
				  ROWS_widget widget)
{
  _p_ROWS_window_data **private;
  _p_ROWS_X_window_data x_data;

  private = (_p_ROWS_window_data **) ROWS_window_cast(widget);
  x_data = (_p_ROWS_X_window_data)(*private)->int_data;

  ROWS_drawable_set_valid(ROWS_child_get_drawable(ROWS_child_cast(widget)), 1);

  return 1;
}

static int  _p_ROWS_window_unmap (XEvent  *event,
				  ROWS_widget widget)
{
  _p_ROWS_window_data **private;
  _p_ROWS_X_window_data x_data;

  private = (_p_ROWS_window_data **) ROWS_window_cast(widget);
  x_data = (_p_ROWS_X_window_data)(*private)->int_data;

  ROWS_drawable_set_valid(ROWS_child_get_drawable(ROWS_child_cast(widget)), 0);

  return 1;
}

static int  _p_ROWS_window_configure(XEvent  *event,
				     ROWS_widget widget)
{
  ROWS_point       size;
  ROWS_box         box;
  ROWS_child_event ev;
  
  box = ROWS_box_cast(widget);

  size.x = PIXEL_TO_MICRONS(event->xconfigure.width);
  size.y = PIXEL_TO_MICRONS(event->xconfigure.height);

  ROWS_box_set_physical(box, &size);
  ROWS_box_set_virtual (box, &size);
  
  ev.event.resize.newsize = size;
  ROWS_child_event_send(ROWS_child_cast(widget),
			ROWS_CHILD_RESIZE,
			&ev);
  
  return 1;
}

static int  _p_ROWS_window_key      (XEvent  *event,
				     ROWS_widget widget)
{
  ROWS_child_event ev;

  ev.event.key.keycode = event->xkey.keycode;
  ev.event.key.state   = event->xkey.state;


  ROWS_child_event_send(ROWS_child_cast(widget),
			ROWS_CHILD_KEY,
			&ev);
  
  return 1;
}

static int  _p_ROWS_window_click    (XEvent  *event,
				     ROWS_widget widget)
{
  ROWS_child_event ev;

  ev.event.click.Xoffset  = PIXEL_TO_MICRONS(event->xbutton.x);
  ev.event.click.Yoffset  = PIXEL_TO_MICRONS(event->xbutton.y);
  
  /* event->xbutton.state? */
  ev.event.click.button   = event->xbutton.button;
  ev.event.click.released = (event->xbutton.type == ButtonRelease);

  ROWS_child_event_send(ROWS_child_cast(widget),
			ROWS_CHILD_CLICK,
			&ev);
  
  return 1;
}


static int  _p_ROWS_window_client   (XEvent  *event,
				     ROWS_widget widget)
{
  Atom deletewin, protocols;
  _p_ROWS_window_data  **private;
  _p_ROWS_X_window_data x_data;

  private = (_p_ROWS_window_data **) ROWS_window_cast(widget);
  x_data = (_p_ROWS_X_window_data)(*private)->int_data;
  deletewin = XInternAtom(x_data->display,
			   "WM_DELETE_WINDOW",
			   1);
  protocols = XInternAtom(x_data->display,
			  "WM_PROTOCOLS",
			   1);
  
  if (event->xclient.type == protocols &&
      event->xclient.data.l[0] == deletewin)
  {
    return 1;
  }
  
  return 0;
}

/*                            ======--------======                           */

/* Threading functions */

#if USE_THREADS

static int _p_window_pred(impulse_event *ev,
			  void          *data)
{
  _p_ROWS_window_data            **private;
  _p_ROWS_X_window_data x_data;

  if (ev->type != IMPULSE_EVENT_GUI)
    return 0;
  
  private = data;
  x_data = (_p_ROWS_X_window_data)(*private)->int_data;

  if (ev->data.gui.object == x_data->window)
    return 1;
  
  return 0;
}

/* FIXME: make a way to cancel this thread before destroying the window */
static void *_p_ROWS_window_thread(void *data)
{
  impulse_event                    im_event;

  for(;;)
  {
    impulse_event_get_next_predicate(&im_event,
				     _p_window_pred,
				     data,
				     0);
    
    window_event(&im_event, data);
  }
}

#endif
