/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * list.h
 * ------
 *
 * Double headed lists
 */

#ifndef __impulse_util_list_h
#define __impulse_util_list_h

#include <impulse/util/iterator.h>

/*                            ======--------======                           */

/* listing data structures */

typedef struct _impulse_list *impulse_list;
typedef void (*impulse_list_cleaner_func)(void *data);

/*                            ======--------======                           */

/* listing functions */

impulse_list impulse_list_create(void);
void impulse_list_destroy(impulse_list list);

void impulse_list_cleaner(impulse_list list,
			   impulse_list_cleaner_func func);
impulse_iterator impulse_list_elements(impulse_list list);

impulse_bool impulse_list_empty(impulse_list list);
impulse_uint impulse_list_count(impulse_list list);

void impulse_list_append(impulse_list list,
			 void *data);
void impulse_list_prepend(impulse_list list,
			  void *data);
void impulse_list_insert(impulse_list list,
			 impulse_uint index,
			 void *data);
void *impulse_list_find(impulse_list list,
			impulse_uint index);
void *impulse_list_remove(impulse_list list,
			  impulse_uint index);
void impulse_list_clear(impulse_list list);
impulse_bool impulse_list_contains(impulse_list list,
				    void *data);

#endif

