/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * main.h
 * ------
 *
 * Impulse program startup and termination functions
 */


#ifndef __impulse_util_main_h
#define __impulse_util_main_h

#include <impulse/util/unistring.h>


enum _impulse_command_type
{
  IMPULSE_COMMAND_TYPE_NONE,
  IMPULSE_COMMAND_TYPE_INTEGER,
  IMPULSE_COMMAND_TYPE_DECIMAL,
  IMPULSE_COMMAND_TYPE_STRING,
  IMPULSE_COMMAND_TYPE_INTEGERS,
  IMPULSE_COMMAND_TYPE_DECIMALS,
  IMPULSE_COMMAND_TYPE_STRINGS,
};

union _impulse_command_value
{
  impulse_int i;
  impulse_double d;
  impulse_unistring s;
};

typedef enum _impulse_command_type impulse_command_type;
typedef union _impulse_command_value *impulse_command_value;

typedef void (*impulse_command_func)(void);


void impulse_main(void);
void impulse_quit(void);
impulse_bool impulse_is_quit(void);

void impulse_init(impulse_uint argc,
		  impulse_char *argv[]);
void impulse_cleanup(void);

void impulse_command_help(void);
impulse_uint impulse_command_parse(void);

void impulse_command_option(impulse_unistring lopt,
			    impulse_unichar sopt,
			    impulse_unistring help);

void impulse_command_option_arg(impulse_unistring lopt,
				impulse_unichar sopt,
				impulse_unistring arg,
				impulse_command_type type,
				impulse_unistring help);

impulse_uint impulse_command_set(impulse_unistring lopt,
				impulse_command_value *values);

void impulse_command_hook(impulse_command_func func);

/* 
 * This interface may change in a future release, so don't go
 * using it too often
 */
impulse_uint impulse_command_args(impulse_uint *argc,
				  impulse_unistring **argv);

#endif __impulse_main_h

/* End of file */
