/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * mapping.h
 * ---------
 *
 * Data structures used to represent mappings between the Unicode 
 * character set and various 8bit character encodings
 */

#ifndef __impulse_util_mapping_h
#define __impulse_util_mapping_h

#include <impulse/util/unistring.h>

/*                            ======--------======                           */

/* Mapping table structures */

enum impulse_unicode_map_type
{
  UNICODE_MAP_SINGLE = 0,
  UNICODE_MAP_MULTIPLE,
  UNICODE_MAP_UNMAPPED
};

/* 8-bit codes */

/* Note that support is provided for multiple-character mappings */

struct _impulse_mapping_from_8bit
{
  struct {
    impulse_ubyte      type; /* See enum impulse_unicode_map_type */
    impulse_unichar   mapchar;
    struct
    {
      impulse_ubyte length;
      impulse_unichar *string;
    } mapstr;
  } mapping[256];
};

struct _impulse_mapping_to_8bit
{
  struct _impulse_mapping_to_8bit_int {
    impulse_ubyte type; /* See enum impulse_unicode_map_type */
    char mapchar;
    struct {
      impulse_ubyte  length;
      char *string;
    } mapstr;
  } *mapping[256];
};

typedef struct _impulse_mapping_to_8bit *impulse_mapping_to_8bit;
typedef struct _impulse_mapping_from_8bit *impulse_mapping_from_8bit;


struct _impulse_mapping
{
  impulse_unistring encoding;
  impulse_mapping_to_8bit to;
  impulse_mapping_from_8bit from;
};

typedef struct _impulse_mapping *impulse_mapping;



/* Mapping table functions */

impulse_mapping impulse_mapping_create(impulse_unistring encoding);
void impulse_mapping_destroy(impulse_mapping map);

#endif
