/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * memory.c
 * --------
 *
 * Memory handling functions
 */

#include "../../../config.h"

#include "impulse/util/memory.h"
#include "impulse/util/error.h"
#include "impulse/util/unistring.h"

#include <stdlib.h>
#include <unistd.h>

#if DEBUG_MCHECK
#include <mcheck.h>
#endif


#if DEBUG_MCHECK
void abortfunc(enum mcheck_status stat)
{
  if (stat == MCHECK_OK ||
      stat == MCHECK_DISABLED)
    return;
  
  write(STDERR_FILENO, "*****\n", 6);
  write(STDERR_FILENO, "***** Impulse Emergency\n", 24);
  write(STDERR_FILENO, "***** Memory corruption\n", 24);
  switch (stat)
  {
   case MCHECK_FREE:
    write(STDERR_FILENO, "***** Block freed twice\n", 24);
    break;
    
   case MCHECK_HEAD:
    write(STDERR_FILENO, "***** Memory before block was clobbered\n", 40);
    break;
    
   case MCHECK_TAIL:
    write(STDERR_FILENO, "***** Memory after block was clobbered\n", 39);
    break;
    
   default:
    write(STDERR_FILENO, "***** Unknown problem\n", 22);
    break;
  }
  write(STDERR_FILENO, "*****\n", 6);
  sleep(60);
  exit(1);
}
#endif

void impulse_mcheck(void)
{
#if DEBUG_MCHECK
  mcheck(abortfunc);
#endif
}


void *impulse_alloc(size_t size)
{
  void *d;

  d = malloc(size);

  if (!d)
  {
    /*
     * While I would love to call our error handling functions now,
     * they would no doubt try to allocate more memory resulting in
     * bad shit happening. (infinite loops for instance)
     */
    write(STDERR_FILENO, "*****\n", 6);
    write(STDERR_FILENO, "***** Impulse Emergency\n",24);
    write(STDERR_FILENO, "***** Out of memory\n", 20);
    write(STDERR_FILENO, "*****\n", 6);
    exit(1);
  }
  return d;
}


void *impulse_realloc(void *old, size_t size)
{
  void *d;
  
  d = realloc(old, size);

  if (!d)
  {
   /*
    * cf. impulse_alloc
    */
    write(STDERR_FILENO, "*****\n", 6);
    write(STDERR_FILENO, "***** Impulse Emergency\n",24);
    write(STDERR_FILENO, "***** Out of memory\n", 20);
    write(STDERR_FILENO, "*****\n", 6);
    exit(1);
  }
  return d;
}
  

void impulse_free(void *old)
{
  if (old)
    free(old);
}


/* End of file */
