/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * unistring.h
 * -----------
 *
 * Functions to manipulate Unicode strings
 */

#ifndef __impulse_util_unistring_h
#define __impulse_util_unistring_h

#include <impulse/util/unichar.h>


struct _impulse_unistring {
  impulse_int length;
  impulse_int capacity;
  impulse_unichar *data;
};

typedef struct _impulse_unistring *impulse_unistring;


extern impulse_unistring impulse_unistring_create_empty(void);
extern impulse_unistring impulse_unistring_create(impulse_unistring src);
extern impulse_unistring impulse_unistring_create_utf(impulse_char *src, 
						      impulse_int bits);
extern impulse_unistring impulse_unistring_create_8bit(impulse_char *src, 
						       impulse_unistring map);
extern void impulse_unistring_destroy(impulse_unistring str);


/* Conversion functions */

char *impulse_unistring_to_utf(impulse_unistring string, 
			       impulse_int bitlength, 
			       impulse_int options);
void impulse_unistring_from_utf(impulse_unistring string, 
				impulse_char *chunk,
				impulse_int bitlength);

char *impulse_unistring_to_8bit(impulse_unistring string, 
				impulse_unistring map);
void impulse_unistring_from_8bit(impulse_unistring string, 
				 impulse_char *chunk, 
				 impulse_unistring map);


void impulse_unistring_decompose(impulse_unistring string);
void impulse_unistring_compose(impulse_unistring string);
void impulse_unistring_bidirection(impulse_unistring string);


/* Manipulation functions */

extern void impulse_unistring_resize(impulse_unistring str,
                                     impulse_int len);
extern void impulse_unistring_trim(impulse_unistring str);
extern void impulse_unistring_substring(impulse_unistring dst,
					impulse_unistring src,
					impulse_int start,
					impulse_int end);

extern void impulse_unistring_assign(impulse_unistring str,
                                     impulse_unistring chunk);
extern void impulse_unistring_append(impulse_unistring str,
                                     impulse_unistring chunk);
extern void impulse_unistring_prepend(impulse_unistring str,
                                      impulse_unistring chunk);
extern void impulse_unistring_insert(impulse_unistring str,
                                     impulse_unistring chunk,
                                     impulse_int offset);

extern void impulse_unistring_assign_utf(impulse_unistring str,
					 impulse_char *chunk,
					 impulse_int bits);
extern void impulse_unistring_append_utf(impulse_unistring str,
					 impulse_char *chunk,
					 impulse_int bits);
extern void impulse_unistring_prepend_utf(impulse_unistring str,
					  impulse_char *chunk,
					  impulse_int bits);
extern void impulse_unistring_insert_utf(impulse_unistring str,
					 impulse_char *chunk,
					 impulse_int bits,
					 impulse_int offset);

extern void impulse_unistring_assign_8bit(impulse_unistring str,
					  impulse_char *chunk,
					  impulse_unistring map);
extern void impulse_unistring_append_8bit(impulse_unistring str,
					  impulse_char *chunk,
					  impulse_unistring map);
extern void impulse_unistring_prepend_8bit(impulse_unistring str,
					   impulse_char *chunk,
					   impulse_unistring map);
extern void impulse_unistring_insert_8bit(impulse_unistring str,
					  impulse_char *chunk,
					  impulse_unistring map,
					  impulse_int offset);

/* Search functions */

extern impulse_int impulse_unistring_find(impulse_unistring str,
					  impulse_unistring chunk);
extern impulse_int impulse_unistring_rfind(impulse_unistring str,
					   impulse_unistring chunk);
extern impulse_int impulse_unistring_find_from(impulse_unistring str,
					       impulse_unistring chunk,
					       impulse_int offset);
extern impulse_int impulse_unistring_rfind_from(impulse_unistring str,
						impulse_unistring chunk,
						impulse_int offset);

extern impulse_int impulse_unistring_find_chr(impulse_unistring str,
					      impulse_unichar c);
extern impulse_int impulse_unistring_rfind_chr(impulse_unistring str,
					       impulse_unichar c);
extern impulse_int impulse_unistring_find_from_chr(impulse_unistring str,
						   impulse_unichar c,
						   impulse_int offset);
extern impulse_int impulse_unistring_rfind_from_chr(impulse_unistring str,
						    impulse_unichar c,
						    impulse_int offset);

/* Comparison functions */

extern impulse_bool impulse_unistring_equal(impulse_unistring str1,
                                            impulse_unistring str2);
extern impulse_int impulse_unistring_compare(impulse_unistring str1,
					     impulse_unistring str2);
extern impulse_int impulse_unistring_compare_case(impulse_unistring str1,
						  impulse_unistring str2);
extern impulse_int impulse_unistring_ncompare(impulse_unistring str1,
					      impulse_unistring str2,
					      impulse_int count);
extern impulse_int impulse_unistring_ncompare_case(impulse_unistring str1,
						   impulse_unistring str2,
						   impulse_int count);

extern impulse_int impulse_unistring_starts(impulse_unistring str,
					    impulse_unistring chunk);
extern impulse_int impulse_unistring_ends(impulse_unistring str,
					  impulse_unistring chunk);


#endif


/* End of file */
