dnl
dnl DEBUG functions
dnl

dnl *****************************************************************
dnl * Initialise DEBUG configuration information
dnl *****************************************************************


AC_DEFUN(IMPULSE_DEBUG_INIT,
[
  DEBUG=0
  AC_ARG_ENABLE(debug, [  --enable-debug          Enable debugging output],
  [
    if test "$enableval" = yes; then
      DEBUG=1
    fi
  ])
])


dnl *****************************************************************
dnl * Check for DEBUG functionality
dnl *****************************************************************

AC_DEFUN(IMPULSE_DEBUG_CHECK,
[
  IMPULSE_MSG_SECTION([Checking debug functionality])
  IMPULSE_DEBUG_CHECK_MCHECK
  IMPULSE_DEBUG_CHECK_GDB
])


dnl *****************************************************************
dnl * Check for a memory corruption checking version of malloc
dnl *****************************************************************


AC_DEFUN(IMPULSE_DEBUG_CHECK_MCHECK,
[
  AC_CHECK_HEADER(mcheck.h,
  [
    AC_CACHE_CHECK([for mcheck], impulse_cv_mcheck,
    [
      AC_TRY_COMPILE([
        #include <mcheck.h>
	#include <stdlib.h>
      ],[
        mcheck(NULL);
      ],[
        AC_TRY_LINK([
	  #include <mcheck.h>
	  #include <stdlib.h>
	],[
	  mcheck(NULL);
	],[
          impulse_cv_mcheck="yes"
	],[
	  impulse_cv_mcheck="no"
	])
      ],[
        impulse_cv_mcheck="no"
      ])
    ])
    
    if test "$impulse_cv_mcheck" = "yes" ; then
      AC_DEFINE(HAVE_DEBUG_MCHECK, 1)
    fi
  ])
])


dnl *****************************************************************
dnl * Check for gdb debugger
dnl *****************************************************************

AC_DEFUN(IMPULSE_DEBUG_CHECK_GDB,
[
  AC_PATH_PROG(GDB, gdb, "none")
  if test "$GDB" != "none" ; then
    AC_DEFINE(HAVE_DEBUG_GDB, 1)
  fi
])


dnl *****************************************************************
dnl * Finish up configuration
dnl *****************************************************************

AC_DEFUN(IMPULSE_DEBUG_FINISH,
[
  if test "$DEBUG" = "1" ; then
    CFLAGS="$CFLAGS -DDEBUG"
  fi
])


dnl *****************************************************************
dnl * Check for a few programs which we need
dnl *****************************************************************

AC_DEFUN(IMPULSE_DEBUG_SUMMARY,
[
])
        
