dnl 
dnl Macros to determine what image libraries available
dnl


dnl *****************************************************************
dnl * Initialise configuration
dnl *****************************************************************

AC_DEFUN(IMPULSE_IMAGE_INIT,
[
])


dnl *****************************************************************
dnl * Check to see if PNG libs are available
dnl *****************************************************************

AC_DEFUN(IMPULSE_IMAGE_CHECK_PNG,
[
  IMPULSE_PARAMS_SAVE
  PNG_LIBS=""
  PNG_LDFLAGS=""
  PNG_CFLAGS=""

dnl  LIBS="$LIBS $GUI_LIBS"
dnl  LDFLAGS="$LDFLAGS $GUI_FLAGS"
dnl  CFLAGS="$CFLAGS $GUI_CFLAGS"
  
  AC_CHECK_LIB(z, deflate,
  [
    AC_CHECK_LIB(png, png_check_sig,
    [
      PNG_LIBS="$PNG_LIBS -lz -lpng"
      AC_DEFINE(HAVE_IMAGES_PNG, 1)
    ])
  ])

  IMPULSE_PARAMS_RESTORE
])


dnl *****************************************************************
dnl * Check to see if JPEG libraries are available
dnl *****************************************************************

AC_DEFUN(IMPULSE_IMAGE_CHECK_JPEG,
[
  IMPULSE_PARAMS_SAVE
  JPEG_LIBS=""
  JPEG_LDFLAGS=""
  JPEG_CFLAGS=""

dnl  LIBS="$LIBS $GUI_LIBS"
dnl  LDFLAGS="$LDFLAGS $GUI_FLAGS"
dnl  CFLAGS="$CFLAGS $GUI_CFLAGS"
  
  AC_CHECK_LIB(jpeg, jpeg_CreateCompress,
  [
    JPEG_LIBS="$JPEG_LIBS -ljpeg"
    AC_DEFINE(HAVE_IMAGES_JPEG, 1)
  ])

  IMPULSE_PARAMS_RESTORE
])


dnl *****************************************************************
dnl * Check to see if XPM libraries are available
dnl *****************************************************************

AC_DEFUN(IMPULSE_IMAGE_CHECK_XPM,
[
  IMPULSE_PARAMS_SAVE
  IMAGE_LIBS=""
  IMAGE_LDFLAGS=""
  IMAGE_CFLAGS=""

dnl  LIBS="$LIBS $GUI_LIBS"
dnl  LDFLAGS="$LDFLAGS $GUI_FLAGS"
dnl  CFLAGS="$CFLAGS $GUI_CFLAGS"
  
  AC_CHECK_HEADER(X11/xpm.h,
  [
    AC_CHECK_LIB(Xpm, XpmReadFileToPixmap,
    [
      XPM_LIBS="$XPM_LIBS -lXpm"
      AC_DEFINE(HAVE_IMAGES_XPM, 1)
    ])
  ])

  IMPULSE_PARAMS_RESTORE
])


dnl *****************************************************************
dnl * Check to see if TIFF libraries are available
dnl *****************************************************************

AC_DEFUN(IMPULSE_IMAGE_CHECK_TIFF,
[
  IMPULSE_PARAMS_SAVE
  TIFF_LIBS=""
  TIFF_LDFLAGS=""
  TIFF_CFLAGS=""

dnl  LIBS="$LIBS $GUI_LIBS"
dnl  LDFLAGS="$LDFLAGS $GUI_FLAGS"
dnl  CFLAGS="$CFLAGS $GUI_CFLAGS"
  
  AC_CHECK_HEADER(tiffio.h,
  [
    AC_CHECK_LIB(tiff, TIFFOpen,
    [
      TIFF_LIBS="$TIFF_LIBS -ltiff"
      AC_DEFINE(HAVE_IMAGES_TIFF, 1)
    ])
  ])

  IMPULSE_PARAMS_RESTORE
])


dnl *****************************************************************
dnl * Check to see what image libraries are available
dnl *****************************************************************

AC_DEFUN(IMPULSE_IMAGE_CHECK,
[
  IMPULSE_MSG_SECTION([Checking image libraries])

  IMPULSE_IMAGE_CHECK_PNG
  IMPULSE_IMAGE_CHECK_JPEG
  IMPULSE_IMAGE_CHECK_XPM
  IMPULSE_IMAGE_CHECK_TIFF
])



dnl *****************************************************************
dnl * Finish up configuration stuff
dnl *****************************************************************


AC_DEFUN(IMPULSE_IMAGE_FINISH,
[
  AC_SUBST(PNG_CFLAGS)
  AC_SUBST(PNG_LDFLAGS)
  AC_SUBST(PNG_LIBS)

  AC_SUBST(JPEG_CFLAGS)
  AC_SUBST(JPEG_LDFLAGS)
  AC_SUBST(JPEG_LIBS)

  AC_SUBST(XPM_CFLAGS)
  AC_SUBST(XPM_LDFLAGS)
  AC_SUBST(XPM_LIBS)

  AC_SUBST(TIFF_CFLAGS)
  AC_SUBST(TIFF_LDFLAGS)
  AC_SUBST(TIFF_LIBS)
])


dnl *****************************************************************
dnl * Print configuration summary
dnl *****************************************************************


AC_DEFUN(IMPULSE_IMAGE_SUMMARY,
[
])


dnl End of file
