dnl
dnl Macro to determine which dynamic library loading subsystem
dnl to use
dnl

dnl *****************************************************************
dnl * Command line arguments for dynamic module interface
dnl *****************************************************************


AC_DEFUN(IMPULSE_MODULE_INIT,
[
  AC_ARG_WITH([module],
  [  --with-modules=[dl/dld/shl/win32]
                          Select dynamic module interface],
  [
    case $with_module in
     dl)
       MODULE_PREFERENCE="DL";;
      
     shl)
       MODULE_PREFERENCE="SHL";;
	
     win32)
       MODULE_PREFERENCE="WIN32";;
	
     dld)
       MODULE_PREFERENCE="DLD";;
	
     *)
       MODULE_PREFERENCE="NONE";;
    esac
  ],[
    MODULE_PREFERENCE="NONE";
  ])
  
  HAVE_MODULES="0"
  MODULE_LIBRARY="none"
])


dnl *****************************************************************
dnl * Check for dynamic module support
dnl *****************************************************************


AC_DEFUN(IMPULSE_MODULE_CHECK,
[
  IMPULSE_MSG_SECTION([Checking dynamic module support])

  if test "$HAVE_MODULES" = "0" ; then
    if test "$MODULE_PREFERENCE" = "NONE" -o "$MODULE_PREFERENCE" = "DL" ; then
      IMPULSE_MODULE_CHECK_DL
    fi
  fi
  if test "$HAVE_MODULES" = "0" ; then
    if test "$MODULE_PREFERENCE" = "NONE" -o "$MODULE_PREFERENCE" = "SHL" ; then
      IMPULSE_MODULE_CHECK_SHL
    fi
  fi
  if test "$HAVE_MODULES" = "0" ; then
    if test "$MODULE_PREFERENCE" = "NONE" -o "$MODULE_PREFERENCE" = "WIN32" ; then
      IMPULSE_MODULE_CHECK_WIN32
    fi
  fi
  if test "$HAVE_MODULES" = "0" ; then
    if test "$MODULE_PREFERENCE" = "NONE" -o "$MODULE_PREFERENCE" = "DLD" ; then
      IMPULSE_MODULE_CHECK_DLD
    fi
  fi
  
  if test "$HAVE_MODULES" = "1" ; then
    AC_DEFINE(HAVE_MODULES, 1)
  fi
])


dnl *****************************************************************
dnl * Check for dlopen module support
dnl *****************************************************************


AC_DEFUN(IMPULSE_MODULE_CHECK_DL,
[
  AC_CHECK_FUNC(dlopen,
  [
    AC_CHECK_HEADER(dlfcn.h,
    [
      HAVE_MODULES="1"
      MODULE_LIBRARY="dl"
      AC_DEFINE(HAVE_MODULE_DL, 1)
      
      AC_CHECK_FUNC(dlerror, AC_DEFINE(HAVE_MODULE_DL_DLERROR, 1))
    ])
  ],[
    AC_CHECK_LIB(dl, dlopen,
    [
      AC_CHECK_HEADER(dlfcn.h,
      [
        HAVE_MODULES="1"
	MODULE_LIBRARY="dl"
	AC_DEFINE(HAVE_MODULE_DL, 1)
	LIBS="$LIBS -ldl"
	
	AC_CHECK_LIB(dl, dlerror, AC_DEFINE(HAVE_MODULE_DL_DLERROR, 1))
      ])
    ])
  ])
])


dnl *****************************************************************
dnl * Check for HPUX shl dynamic module support
dnl *****************************************************************


AC_DEFUN(IMPULSE_MODULE_CHECK_SHL,
[
  AC_CHECK_LIB(dld, shl_load,
  [
    AC_CHECK_HEADER(dl.h,
    [
      HAVE_MODULES="1"
      MODULE_LIBRARY="shl"
      AC_DEFINE(HAVE_MODULE_SHL, 1)
      
      LIBS="$LIBS -ldld"
    ])
  ])
])



dnl *****************************************************************
dnl * Check for Win32 dynamic module support
dnl *****************************************************************


AC_DEFUN(IMPULSE_MODULE_CHECK_WIN32,
[
  AC_CHECK_FUNC(LoadLibrary,
  [
    AC_CHECK_HEADER(windows.h, 
    [
      HAVE_MODULES="1"
      MODULE_LIBRARY="win32"
      AC_DEFINE(HAVE_MODULE_WIN32)
    ])
  ])
])


dnl *****************************************************************
dnl * Check for Dld dynamic module support
dnl *****************************************************************


AC_DEFUN(IMPULSE_MODULE_CHECK_DLD,
[
  AC_CHECK_LIB(dld, dld_init,
  [
    AC_CHECK_HEADER(dld.h,
    [
      HAVE_MODULES="1"
      MODULE_LIBRARY="dld"
      AC_DEFINE(HAVE_MODULE_DLD)
      LIBS="$LIBS -ldld"
    ])
  ])
])


dnl *****************************************************************
dnl * Substitute in all our variables
dnl *****************************************************************


AC_DEFUN(IMPULSE_MODULE_FINISH,
[
])


dnl *****************************************************************
dnl * Finish up configuration
dnl *****************************************************************


AC_DEFUN(IMPULSE_MODULE_SUMMARY,
[
  IMPULSE_ECHO(["Modules library: $MODULE_LIBRARY"])
])

dnl End of file
