dnl
dnl Check out compiler types
dnl

dnl *****************************************************************
dnl * Finish up configuration
dnl *****************************************************************

AC_DEFUN(IMPULSE_TYPE_INIT,
[
  BYTE_TYPE="int8_t"
  UBYTE_TYPE="uint8_t"
  
  SHORT_TYPE="int16_t"
  USHORT_TYPE="uint16_t"
  
  INT_TYPE="int32_t"
  UINT_TYPE="uint32_t"
  
  LONG_TYPE="int64_t"
  ULONG_TYPE="uint64_t"
  
  BYTE_MIN="INT8_MIN"
  BYTE_MAX="INT8_MAX"
  UBYTE_MAX="UINT8_MAX"
 
  SHORT_MIN="INT16_MIN"
  SHORT_MAX="INT16_MAX"
  USHORT_MAX="UINT16_MAX"

  INT_MIN="INT32_MIN"
  INT_MAX="INT32_MAX"
  UINT_MAX="UINT32_MAX"
  
  LONG_MIN="INT64_MIN"
  LONG_MAX="INT64_MAX"
  ULONG_MAX="UINT64_MAX"
  
  IMPULSE_TYPES_INCLUDE=""
  
  IMPULSE_CONST=""
  IMPULSE_INLINE=""
  IMPULSE_ENDIAN=""
])

dnl *****************************************************************
dnl * Finish up configuration
dnl *****************************************************************


AC_DEFUN(IMPULSE_TYPE_CHECK,
[
  IMPULSE_MSG_SECTION([Checking compiler characteristics])
  IMPULSE_PARAMS_SAVE
  
  AC_TYPE_SIZE_T
  AC_C_CONST
  AC_C_BIGENDIAN
  AC_C_INLINE
  
  case "$ac_cv_c_inline" in
    inline | yes) IMPULSE_INLINE="inline";;
    no) IMPULSE_INLINE="" ;;
    *)  IMPULSE_INLINE="$ac_cv_c_inline";;
  esac

  if test "$ac_cv_c_const" = "yes" ; then
    IMPULSE_CONST="const"
  else
    IMPULSE_CONST=""
  fi

  if test "$ac_cv_c_bigendian" = "yes" ; then
    IMPULSE_ENDIAN="1234"
  else
    IMPULSE_ENDIAN="4321"
  fi

  dnl Long doubles are just too damn f**ked up to use reliably
  dnl Gcc claims it is a 12 byte type, yet certain 80387 instructions
  dnl used for manipulating them only move 10 bytes - Grrr..
  dnl As for Sparcs.... - don't get me started on that
  dnl
  dnl AC_C_LONG_DOUBLE
  
  
  AC_CHECK_HEADER(inttypes.h,
  [
    IMPULSE_TYPES_INCLUDE="#include <inttypes.h>"
  ],[
    AC_CHECK_HEADER(stdint.h,
    [
      IMPULSE_TYPES_INCLUDE="#include <stdint.h>"
    ],[
      # FIXME: there's an alternative to long long in some compilers... what?
  
      AC_CACHE_CHECK([for long long int], impulse_cv_longlong,
      [
        AC_TRY_COMPILE( , long long int foo,
          impulse_cv_longlong="yes",
	  impulse_cv_longlong="no")
      ])
    
  
      # Check sizes of types (make safe-ish assumptions for irritating x-compiles)
      AC_CHECK_SIZEOF(unsigned char, 1)
      AC_CHECK_SIZEOF(short int, 2)
      AC_CHECK_SIZEOF(int,       2)
      AC_CHECK_SIZEOF(long int,  4)
      AC_CHECK_SIZEOF(void *,    8)
      if test "$impulse_cv_longlong" = "yes"; then
        AC_CHECK_SIZEOF(long long int, 8)
      else
        ac_cv_sizeof_long_long_int="0"
      fi
      
      
      # Work out 8, 16, 32, 64 bit values where available
      
      
      BYTE_MIN="(-128)"
      BYTE_MAX="(127)"
      UBYTE_MAX="(255)"
      
      case 1 in
      $ac_cv_sizeof_unsigned_char)
        BYTE_TYPE="signed char"
        UBYTE_TYPE="unsigned char";;
      
      $ac_cv_sizeof_short_int) 
        BYTE_TYPE="signed short int"
        UBYTE_TYPE="unsigned short int";;
      
      *)
        BYTE_TYPE="signed char"
        UBYTE_TYPE="unsigned char";;
      esac
      
      
      SHORT_MIN="(-32767-1)"
      SHORT_MAX="(32767)"
      USHORT_MAX="(65535U)"

      case 2 in
      $ac_cv_sizeof_short_int)
        SHORT_TYPE="signed short int"
        USHORT_TYPE="unsigned short int";;
      
      $ac_cv_sizeof_int)
        SHORT_TYPE="signed int"
        USHORT_TYPE="unsigned int";;
      
      *)
        SHORT_TYPE="signed short int"
        USHORT_TYPE="unsigned short int";;
      esac
      
      
      INT_MIN="(-2147483647-1)"
      INT_MAX="(2147483647)"
      UINT_MAX="(4294967295U)"

      case 4 in
      $ac_cv_sizeof_short_int)
        INT_TYPE="signed short int"
        UINT_TYPE="unsigned short int";;
      
      $ac_cv_sizeof_int)
        INT_TYPE="signed int"
        UINT_TYPE="unsigned int";;
      
      $ac_cv_sizeof_long_int)
        INT_TYPE="signed long int"
        UINT_TYPE="unsigned long int";;
      
      *)
        INT_TYPE="signed int"
        UINT_TYPE="unsigned int";;
      esac
      
      
      LONG_MIN="(-9223372036854775807LL-1)"
      LONG_MAX="(9223372036854775807LL)"
      ULONG_MAX="(18446744073709551615ULL)"
      
      case 8 in
      $ac_cv_sizeof_int)
        LONG_TYPE="signed int"
        ULONG_TYPE="unsigned int";;
      
      $ac_cv_sizeof_long_int)
        LONG_TYPE="signed long int"
        ULONG_TYPE="unsigned long int";;
      
      $ac_cv_sizeof_long_long_int)
        LONG_TYPE="signed long long int"
        ULONG_TYPE="unsigned long long int";;
      
      *)
        LONG_TYPE="signed long int"
        ULONG_TYPE="unsigned long int";;
      esac
      
      IMPULSE_TYPES_INCLUDE=""
    ])
  ])
  
  BOOL_TYPE="impulse_byte"
  CHAR_TYPE="char"
  UNICHAR_TYPE="impulse_ushort"
  
  FLOAT_TYPE="float"
  DOUBLE_TYPE="double"
  
  IMPULSE_PARAMS_RESTORE
])


dnl *****************************************************************
dnl * Finish up configuration
dnl *****************************************************************


AC_DEFUN(IMPULSE_TYPE_FINISH,
[
  AC_SUBST(IMPULSE_TYPES_INCLUDE)
  
  AC_SUBST(BYTE_TYPE)
  AC_SUBST(SHORT_TYPE)
  AC_SUBST(INT_TYPE)
  AC_SUBST(LONG_TYPE)
  
  AC_SUBST(UBYTE_TYPE)
  AC_SUBST(USHORT_TYPE)
  AC_SUBST(UINT_TYPE)
  AC_SUBST(ULONG_TYPE)
  
  AC_SUBST(CHAR_TYPE)
  AC_SUBST(UNICHAR_TYPE)
  AC_SUBST(BOOL_TYPE)
  AC_SUBST(FLOAT_TYPE)
  AC_SUBST(DOUBLE_TYPE)
  
  AC_SUBST(BYTE_MIN)
  AC_SUBST(SHORT_MIN)
  AC_SUBST(INT_MIN)
  AC_SUBST(LONG_MIN)

  AC_SUBST(BYTE_MAX)
  AC_SUBST(SHORT_MAX)
  AC_SUBST(INT_MAX)
  AC_SUBST(LONG_MAX)

  AC_SUBST(UBYTE_MAX)
  AC_SUBST(USHORT_MAX)
  AC_SUBST(UINT_MAX)
  AC_SUBST(ULONG_MAX)

  AC_SUBST(IMPULSE_INLINE)
  AC_SUBST(IMPULSE_CONST)
  AC_SUBST(IMPULSE_ENDIAN)
])


dnl *****************************************************************
dnl * Print configuration summary
dnl *****************************************************************


AC_DEFUN(IMPULSE_TYPE_SUMMARY,
[
  IMPULSE_MSG_SECTION([Compiler characteristics])
  IMPULSE_ECHO(["  8-bit integer: $BYTE_TYPE"])
  IMPULSE_ECHO([" 16-bit integer: $SHORT_TYPE"])
  IMPULSE_ECHO([" 32-bit integer: $INT_TYPE"])
  IMPULSE_ECHO([" 64-bit integer: $LONG_TYPE"])
])

dnl End of file
